@@ -144,6 +144,9 @@ type InfluxdOpts struct {
 	TracingType       string
 	ReportingDisabled bool
 
+	PIDFile          string
+	OverwritePIDFile bool
+
 	AssetsPath string
 	BoltPath   string
 	SqLitePath string
@@ -213,6 +216,9 @@ func NewOpts(viper *viper.Viper) *InfluxdOpts {
 		FluxLogEnabled:    false,
 		ReportingDisabled: false,
 
+		PIDFile:          "",
+		OverwritePIDFile: false,
+
 		BoltPath:   filepath.Join(dir, bolt.DefaultFilename),
 		SqLitePath: filepath.Join(dir, sqlite.DefaultFilename),
 		EnginePath: filepath.Join(dir, "engine"),
@@ -325,6 +331,18 @@ func (o *InfluxdOpts) BindCliOpts() []cli.Opt {
 			Default: o.ReportingDisabled,
 			Desc:    "disable sending telemetry data to https://telemetry.influxdata.com every 8 hours",
 		},
+		{
+			DestP:   &o.PIDFile,
+			Flag:    "pid-file",
+			Default: o.PIDFile,
+			Desc:    "write process ID to a file",
+		},
+		{
+			DestP:   &o.OverwritePIDFile,
+			Flag:    "overwrite-pid-file",
+			Default: o.OverwritePIDFile,
+			Desc:    "overwrite PID file if it already exists instead of exiting",
+		},
 		{
 			DestP:   &o.SessionLength,
 			Flag:    "session-length",
@@ -513,6 +531,11 @@ func (o *InfluxdOpts) BindCliOpts() []cli.Opt {
 			Default: o.StorageConfig.Data.WALMaxWriteDelay,
 			Desc:    "The max amount of time a write will wait when the WAL already has `storage-wal-max-concurrent-writes` active writes. Set to 0 to disable the timeout.",
 		},
+		{
+			DestP: &o.StorageConfig.Data.WALFlushOnShutdown,
+			Flag:  "storage-wal-flush-on-shutdown",
+			Desc:  "Flushes and clears the WAL on shutdown",
+		},
 		{
 			DestP: &o.StorageConfig.Data.ValidateKeys,
 			Flag:  "storage-validate-keys",
