@@ -1,4 +1,4 @@
-// Package report reports statistics about TSM files.
+// Package reporthelper reports statistics about TSM files.
 package reporthelper
 
 import (
@@ -9,7 +9,7 @@ import (
 	"strconv"
 	"strings"
 
-	"github.com/influxdata/influxdb/tsdb/engine/tsm1"
+	"github.com/influxdata/influxdb/v2/tsdb/engine/tsm1"
 )
 
 func IsShardDir(dir string) error {
