@@ -3,7 +3,7 @@ package tsm1
 // Compactions are the process of creating read-optimized TSM files.
 // The files are created by converting write-optimized WAL entries
 // to read-optimized TSM format.  They can also be created from existing
-// TSM files when there are tombstone records that neeed to be removed, points
+// TSM files when there are tombstone records that need to be removed, points
 // that were overwritten by later writes and need to updated, or multiple
 // smaller TSM files need to be merged to reduce file counts and improve
 // compression ratios.
@@ -14,6 +14,7 @@ package tsm1
 
 import (
 	"bytes"
+	"errors"
 	"fmt"
 	"io"
 	"math"
@@ -26,11 +27,13 @@ import (
 	"sync/atomic"
 	"time"
 
-	"github.com/influxdata/influxdb/pkg/limiter"
-	"github.com/influxdata/influxdb/tsdb"
+	"github.com/influxdata/influxdb/v2/pkg/limiter"
+	"github.com/influxdata/influxdb/v2/tsdb"
+	"go.uber.org/zap"
 )
 
 const maxTSMFileSize = uint32(2048 * 1024 * 1024) // 2GB
+const logEvery = 2 * DefaultSegmentSize
 
 const (
 	// CompactionTempExtension is the extension used for temporary files created during compaction.
@@ -38,6 +41,10 @@ const (
 
 	// TSMFileExtension is the extension used for TSM files.
 	TSMFileExtension = "tsm"
+
+	// DefaultMaxSavedErrors is the number of errors that are stored by a TSMBatchKeyReader before
+	// subsequent errors are discarded
+	DefaultMaxSavedErrors = 100
 )
 
 var (
@@ -87,11 +94,11 @@ type CompactionGroup []string
 // CompactionPlanner determines what TSM files and WAL segments to include in a
 // given compaction run.
 type CompactionPlanner interface {
-	Plan(lastWrite time.Time) []CompactionGroup
-	PlanLevel(level int) []CompactionGroup
-	PlanOptimize() []CompactionGroup
+	Plan(lastWrite time.Time) ([]CompactionGroup, int64)
+	PlanLevel(level int) ([]CompactionGroup, int64)
+	PlanOptimize() ([]CompactionGroup, int64)
 	Release(group []CompactionGroup)
-	FullyCompacted() bool
+	FullyCompacted() (bool, string)
 
 	// ForceFull causes the planner to return a full compaction plan the next
 	// time Plan() is called if there are files that could be compacted.
@@ -110,7 +117,7 @@ type DefaultPlanner struct {
 	// compactFullWriteColdDuration specifies the length of time after
 	// which if no writes have been committed to the WAL, the engine will
 	// do a full compaction of the TSM files in this shard. This duration
-	// should always be greater than the CacheFlushWriteColdDuraion
+	// should always be greater than the CacheFlushWriteColdDuration
 	compactFullWriteColdDuration time.Duration
 
 	// lastPlanCheck is the last time Plan was called
@@ -211,9 +218,15 @@ func (c *DefaultPlanner) ParseFileName(path string) (int, int, error) {
 }
 
 // FullyCompacted returns true if the shard is fully compacted.
-func (c *DefaultPlanner) FullyCompacted() bool {
+func (c *DefaultPlanner) FullyCompacted() (bool, string) {
 	gens := c.findGenerations(false)
-	return len(gens) <= 1 && !gens.hasTombstones()
+	if len(gens) > 1 {
+		return false, "not fully compacted and not idle because of more than one generation"
+	} else if gens.hasTombstones() {
+		return false, "not fully compacted and not idle because of tombstones"
+	} else {
+		return true, ""
+	}
 }
 
 // ForceFull causes the planner to return a full compaction plan the next time
@@ -226,13 +239,13 @@ func (c *DefaultPlanner) ForceFull() {
 }
 
 // PlanLevel returns a set of TSM files to rewrite for a specific level.
-func (c *DefaultPlanner) PlanLevel(level int) []CompactionGroup {
+func (c *DefaultPlanner) PlanLevel(level int) ([]CompactionGroup, int64) {
 	// If a full plan has been requested, don't plan any levels which will prevent
 	// the full plan from acquiring them.
 	c.mu.RLock()
 	if c.forceFull {
 		c.mu.RUnlock()
-		return nil
+		return nil, 0
 	}
 	c.mu.RUnlock()
 
@@ -244,7 +257,7 @@ func (c *DefaultPlanner) PlanLevel(level int) []CompactionGroup {
 	// If there is only one generation and no tombstones, then there's nothing to
 	// do.
 	if len(generations) <= 1 && !generations.hasTombstones() {
-		return nil
+		return nil, 0
 	}
 
 	// Group each generation by level such that two adjacent generations in the same
@@ -254,8 +267,8 @@ func (c *DefaultPlanner) PlanLevel(level int) []CompactionGroup {
 	for i := 0; i < len(generations); i++ {
 		cur := generations[i]
 
-		// See if this generation is orphan'd which would prevent it from being further
-		// compacted until a final full compactin runs.
+		// See if this generation is orphaned which would prevent it from being further
+		// compacted until a final full compaction runs.
 		if i < len(generations)-1 {
 			if cur.level() < generations[i+1].level() {
 				currentGen = append(currentGen, cur)
@@ -313,22 +326,22 @@ func (c *DefaultPlanner) PlanLevel(level int) []CompactionGroup {
 	}
 
 	if !c.acquire(cGroups) {
-		return nil
+		return nil, int64(len(cGroups))
 	}
 
-	return cGroups
+	return cGroups, int64(len(cGroups))
 }
 
 // PlanOptimize returns all TSM files if they are in different generations in order
 // to optimize the index across TSM files.  Each returned compaction group can be
 // compacted concurrently.
-func (c *DefaultPlanner) PlanOptimize() []CompactionGroup {
+func (c *DefaultPlanner) PlanOptimize() ([]CompactionGroup, int64) {
 	// If a full plan has been requested, don't plan any levels which will prevent
 	// the full plan from acquiring them.
 	c.mu.RLock()
 	if c.forceFull {
 		c.mu.RUnlock()
-		return nil
+		return nil, 0
 	}
 	c.mu.RUnlock()
 
@@ -340,7 +353,7 @@ func (c *DefaultPlanner) PlanOptimize() []CompactionGroup {
 	// If there is only one generation and no tombstones, then there's nothing to
 	// do.
 	if len(generations) <= 1 && !generations.hasTombstones() {
-		return nil
+		return nil, 0
 	}
 
 	// Group each generation by level such that two adjacent generations in the same
@@ -356,7 +369,7 @@ func (c *DefaultPlanner) PlanOptimize() []CompactionGroup {
 		}
 
 		// See if this generation is orphan'd which would prevent it from being further
-		// compacted until a final full compactin runs.
+		// compacted until a final full compaction runs.
 		if i < len(generations)-1 {
 			if cur.level() < generations[i+1].level() {
 				currentGen = append(currentGen, cur)
@@ -405,15 +418,15 @@ func (c *DefaultPlanner) PlanOptimize() []CompactionGroup {
 	}
 
 	if !c.acquire(cGroups) {
-		return nil
+		return nil, int64(len(cGroups))
 	}
 
-	return cGroups
+	return cGroups, int64(len(cGroups))
 }
 
 // Plan returns a set of TSM files to rewrite for level 4 or higher.  The planning returns
 // multiple groups if possible to allow compactions to run concurrently.
-func (c *DefaultPlanner) Plan(lastWrite time.Time) []CompactionGroup {
+func (c *DefaultPlanner) Plan(lastWrite time.Time) ([]CompactionGroup, int64) {
 	generations := c.findGenerations(true)
 
 	c.mu.RLock()
@@ -463,19 +476,19 @@ func (c *DefaultPlanner) Plan(lastWrite time.Time) []CompactionGroup {
 
 		// Make sure we have more than 1 file and more than 1 generation
 		if len(tsmFiles) <= 1 || genCount <= 1 {
-			return nil
+			return nil, 0
 		}
 
 		group := []CompactionGroup{tsmFiles}
 		if !c.acquire(group) {
-			return nil
+			return nil, int64(len(group))
 		}
-		return group
+		return group, int64(len(group))
 	}
 
 	// don't plan if nothing has changed in the filestore
 	if c.lastPlanCheck.After(c.FileStore.LastModified()) && !generations.hasTombstones() {
-		return nil
+		return nil, 0
 	}
 
 	c.lastPlanCheck = time.Now()
@@ -483,7 +496,7 @@ func (c *DefaultPlanner) Plan(lastWrite time.Time) []CompactionGroup {
 	// If there is only one generation, return early to avoid re-compacting the same file
 	// over and over again.
 	if len(generations) <= 1 && !generations.hasTombstones() {
-		return nil
+		return nil, 0
 	}
 
 	// Need to find the ending point for level 4 files.  They will be the oldest files. We scan
@@ -576,7 +589,7 @@ func (c *DefaultPlanner) Plan(lastWrite time.Time) []CompactionGroup {
 	}
 
 	if len(groups) == 0 {
-		return nil
+		return nil, 0
 	}
 
 	// With the groups, we need to evaluate whether the group as a whole can be compacted
@@ -604,9 +617,9 @@ func (c *DefaultPlanner) Plan(lastWrite time.Time) []CompactionGroup {
 	}
 
 	if !c.acquire(tsmFiles) {
-		return nil
+		return nil, int64(len(tsmFiles))
 	}
-	return tsmFiles
+	return tsmFiles, int64(len(tsmFiles))
 }
 
 // findGenerations groups all the TSM files by generation based
@@ -699,7 +712,7 @@ type Compactor struct {
 
 	FileStore interface {
 		NextGeneration() int
-		TSMReader(path string) *TSMReader
+		TSMReader(path string) (*TSMReader, error)
 	}
 
 	// RateLimit is the limit for disk writes for all concurrent compactions.
@@ -818,7 +831,7 @@ func (c *Compactor) EnableCompactions() {
 }
 
 // WriteSnapshot writes a Cache snapshot to one or more new TSM files.
-func (c *Compactor) WriteSnapshot(cache *Cache) ([]string, error) {
+func (c *Compactor) WriteSnapshot(cache *Cache, logger *zap.Logger) ([]string, error) {
 	c.mu.RLock()
 	enabled := c.snapshotsEnabled
 	intC := c.snapshotsInterrupt
@@ -834,7 +847,7 @@ func (c *Compactor) WriteSnapshot(cache *Cache) ([]string, error) {
 	// Enable throttling if we have lower cardinality or snapshots are going fast.
 	throttle := card < 3e6 && c.snapshotLatencies.avg() < 15*time.Second
 
-	// Write snapshost concurrently if cardinality is relatively high.
+	// Write snapshot concurrently if cardinality is relatively high.
 	concurrency := card / 2e6
 	if concurrency < 1 {
 		concurrency = 1
@@ -857,7 +870,7 @@ func (c *Compactor) WriteSnapshot(cache *Cache) ([]string, error) {
 	for i := 0; i < concurrency; i++ {
 		go func(sp *Cache) {
 			iter := NewCacheKeyIterator(sp, tsdb.DefaultMaxPointsPerBlock, intC)
-			files, err := c.writeNewFiles(c.FileStore.NextGeneration(), 0, nil, iter, throttle)
+			files, err := c.writeNewFiles(c.FileStore.NextGeneration(), 0, nil, iter, throttle, logger)
 			resC <- res{files: files, err: err}
 
 		}(splits[i])
@@ -891,7 +904,7 @@ func (c *Compactor) WriteSnapshot(cache *Cache) ([]string, error) {
 }
 
 // compact writes multiple smaller TSM files into 1 or more larger files.
-func (c *Compactor) compact(fast bool, tsmFiles []string) ([]string, error) {
+func (c *Compactor) compact(fast bool, tsmFiles []string, logger *zap.Logger) ([]string, error) {
 	size := c.Size
 	if size <= 0 {
 		size = tsdb.DefaultMaxPointsPerBlock
@@ -930,7 +943,10 @@ func (c *Compactor) compact(fast bool, tsmFiles []string) ([]string, error) {
 		default:
 		}
 
-		tr := c.FileStore.TSMReader(file)
+		tr, err := c.FileStore.TSMReader(file)
+		if err != nil {
+			return nil, errCompactionAborted{fmt.Errorf("error creating reader for %q: %w", file, err)}
+		}
 		if tr == nil {
 			// This would be a bug if this occurred as tsmFiles passed in should only be
 			// assigned to one compaction at any one time.  A nil tr would mean the file
@@ -942,19 +958,20 @@ func (c *Compactor) compact(fast bool, tsmFiles []string) ([]string, error) {
 	}
 
 	if len(trs) == 0 {
+		logger.Debug("No input files")
 		return nil, nil
 	}
 
-	tsm, err := NewTSMBatchKeyIterator(size, fast, intC, tsmFiles, trs...)
+	tsm, err := NewTSMBatchKeyIterator(size, fast, DefaultMaxSavedErrors, intC, tsmFiles, trs...)
 	if err != nil {
 		return nil, err
 	}
 
-	return c.writeNewFiles(maxGeneration, maxSequence, tsmFiles, tsm, true)
+	return c.writeNewFiles(maxGeneration, maxSequence, tsmFiles, tsm, true, logger)
 }
 
 // CompactFull writes multiple smaller TSM files into 1 or more larger files.
-func (c *Compactor) CompactFull(tsmFiles []string) ([]string, error) {
+func (c *Compactor) CompactFull(tsmFiles []string, logger *zap.Logger) ([]string, error) {
 	c.mu.RLock()
 	enabled := c.compactionsEnabled
 	c.mu.RUnlock()
@@ -968,7 +985,7 @@ func (c *Compactor) CompactFull(tsmFiles []string) ([]string, error) {
 	}
 	defer c.remove(tsmFiles)
 
-	files, err := c.compact(false, tsmFiles)
+	files, err := c.compact(false, tsmFiles, logger)
 
 	// See if we were disabled while writing a snapshot
 	c.mu.RLock()
@@ -986,7 +1003,7 @@ func (c *Compactor) CompactFull(tsmFiles []string) ([]string, error) {
 }
 
 // CompactFast writes multiple smaller TSM files into 1 or more larger files.
-func (c *Compactor) CompactFast(tsmFiles []string) ([]string, error) {
+func (c *Compactor) CompactFast(tsmFiles []string, logger *zap.Logger) ([]string, error) {
 	c.mu.RLock()
 	enabled := c.compactionsEnabled
 	c.mu.RUnlock()
@@ -1000,7 +1017,7 @@ func (c *Compactor) CompactFast(tsmFiles []string) ([]string, error) {
 	}
 	defer c.remove(tsmFiles)
 
-	files, err := c.compact(true, tsmFiles)
+	files, err := c.compact(true, tsmFiles, logger)
 
 	// See if we were disabled while writing a snapshot
 	c.mu.RLock()
@@ -1031,7 +1048,7 @@ func (c *Compactor) removeTmpFiles(files []string) error {
 
 // writeNewFiles writes from the iterator into new TSM files, rotating
 // to a new file once it has reached the max TSM file size.
-func (c *Compactor) writeNewFiles(generation, sequence int, src []string, iter KeyIterator, throttle bool) ([]string, error) {
+func (c *Compactor) writeNewFiles(generation, sequence int, src []string, iter KeyIterator, throttle bool, logger *zap.Logger) ([]string, error) {
 	// These are the new TSM files written
 	var files []string
 
@@ -1040,16 +1057,19 @@ func (c *Compactor) writeNewFiles(generation, sequence int, src []string, iter K
 
 		// New TSM files are written to a temp file and renamed when fully completed.
 		fileName := filepath.Join(c.Dir, c.formatFileName(generation, sequence)+"."+TSMFileExtension+"."+TmpTSMFileExtension)
+		logger.Debug("Compacting files", zap.Int("file_count", len(src)), zap.String("output_file", fileName))
 
 		// Write as much as possible to this file
-		err := c.write(fileName, iter, throttle)
+		err := c.write(fileName, iter, throttle, logger)
 
 		// We've hit the max file limit and there is more to write.  Create a new file
 		// and continue.
 		if err == errMaxFileExceeded || err == ErrMaxBlocksExceeded {
 			files = append(files, fileName)
+			logger.Debug("file size or block count exceeded, opening another output file", zap.String("output_file", fileName))
 			continue
 		} else if err == ErrNoValues {
+			logger.Debug("Dropping empty file", zap.String("output_file", fileName))
 			// If the file only contained tombstoned entries, then it would be a 0 length
 			// file that we can drop.
 			if err := os.RemoveAll(fileName); err != nil {
@@ -1061,16 +1081,15 @@ func (c *Compactor) writeNewFiles(generation, sequence int, src []string, iter K
 			// planner keeps track of which files are assigned to compaction plans now.
 			return nil, err
 		} else if err != nil {
+			// We hit an error and didn't finish the compaction.  Abort.
 			// Remove any tmp files we already completed
+			// discard later errors to return the first one from the write() call
 			for _, f := range files {
-				if err := os.RemoveAll(f); err != nil {
-					return nil, err
-				}
-			}
-			// We hit an error and didn't finish the compaction.  Remove the temp file and abort.
-			if err := os.RemoveAll(fileName); err != nil {
-				return nil, err
+				_ = os.RemoveAll(f)
 			}
+			// Remove the temp file
+			// discard later errors to return the first one from the write() call
+			_ = os.RemoveAll(fileName)
 			return nil, err
 		}
 
@@ -1081,7 +1100,7 @@ func (c *Compactor) writeNewFiles(generation, sequence int, src []string, iter K
 	return files, nil
 }
 
-func (c *Compactor) write(path string, iter KeyIterator, throttle bool) (err error) {
+func (c *Compactor) write(path string, iter KeyIterator, throttle bool, logger *zap.Logger) (err error) {
 	fd, err := os.OpenFile(path, os.O_CREATE|os.O_RDWR|os.O_EXCL, 0666)
 	if err != nil {
 		return errCompactionInProgress{err: err}
@@ -1134,10 +1153,11 @@ func (c *Compactor) write(path string, iter KeyIterator, throttle bool) (err err
 		}
 
 		if err != nil {
-			w.Remove()
+			_ = w.Remove()
 		}
 	}()
 
+	lastLogSize := w.Size()
 	for iter.Next() {
 		c.mu.RLock()
 		enabled := c.snapshotsEnabled || c.compactionsEnabled
@@ -1176,6 +1196,9 @@ func (c *Compactor) write(path string, iter KeyIterator, throttle bool) (err err
 			}
 
 			return errMaxFileExceeded
+		} else if (w.Size() - lastLogSize) > logEvery {
+			logger.Debug("Compaction progress", zap.String("output_file", path), zap.Uint32("size", w.Size()))
+			lastLogSize = w.Size()
 		}
 	}
 
@@ -1188,6 +1211,7 @@ func (c *Compactor) write(path string, iter KeyIterator, throttle bool) (err err
 	if err := w.WriteIndex(); err != nil {
 		return err
 	}
+	logger.Debug("Compaction finished", zap.String("output_file", path), zap.Uint32("size", w.Size()))
 	return nil
 }
 
@@ -1246,61 +1270,6 @@ func (t TSMErrors) Error() string {
 	return strings.Join(e, ", ")
 }
 
-// tsmKeyIterator implements the KeyIterator for set of TSMReaders.  Iteration produces
-// keys in sorted order and the values between the keys sorted and deduped.  If any of
-// the readers have associated tombstone entries, they are returned as part of iteration.
-type tsmKeyIterator struct {
-	// readers is the set of readers it produce a sorted key run with
-	readers []*TSMReader
-
-	// values is the temporary buffers for each key that is returned by a reader
-	values map[string][]Value
-
-	// pos is the current key position within the corresponding readers slice.  A value of
-	// pos[0] = 1, means the reader[0] is currently at key 1 in its ordered index.
-	pos []int
-
-	// TSMError wraps any error we received while iterating values.
-	errs TSMErrors
-
-	// indicates whether the iterator should choose a faster merging strategy over a more
-	// optimally compressed one.  If fast is true, multiple blocks will just be added as is
-	// and not combined.  In some cases, a slower path will need to be utilized even when
-	// fast is true to prevent overlapping blocks of time for the same key.
-	// If false, the blocks will be decoded and duplicated (if needed) and
-	// then chunked into the maximally sized blocks.
-	fast bool
-
-	// size is the maximum number of values to encode in a single block
-	size int
-
-	// key is the current key lowest key across all readers that has not be fully exhausted
-	// of values.
-	key []byte
-	typ byte
-
-	iterators []*BlockIterator
-	blocks    blocks
-
-	buf []blocks
-
-	// mergeValues are decoded blocks that have been combined
-	mergedFloatValues    FloatValues
-	mergedIntegerValues  IntegerValues
-	mergedUnsignedValues UnsignedValues
-	mergedBooleanValues  BooleanValues
-	mergedStringValues   StringValues
-
-	// merged are encoded blocks that have been combined or used as is
-	// without decode
-	merged    blocks
-	interrupt chan struct{}
-}
-
-func (t *tsmKeyIterator) AppendError(err error) {
-	t.errs = append(t.errs, err)
-}
-
 type block struct {
 	key              []byte
 	minTime, maxTime int64
@@ -1353,242 +1322,6 @@ func (a blocks) Less(i, j int) bool {
 
 func (a blocks) Swap(i, j int) { a[i], a[j] = a[j], a[i] }
 
-// NewTSMKeyIterator returns a new TSM key iterator from readers.
-// size indicates the maximum number of values to encode in a single block.
-func NewTSMKeyIterator(size int, fast bool, interrupt chan struct{}, readers ...*TSMReader) (KeyIterator, error) {
-	var iter []*BlockIterator
-	for _, r := range readers {
-		iter = append(iter, r.BlockIterator())
-	}
-
-	return &tsmKeyIterator{
-		readers:   readers,
-		values:    map[string][]Value{},
-		pos:       make([]int, len(readers)),
-		size:      size,
-		iterators: iter,
-		fast:      fast,
-		buf:       make([]blocks, len(iter)),
-		interrupt: interrupt,
-	}, nil
-}
-
-func (k *tsmKeyIterator) hasMergedValues() bool {
-	return len(k.mergedFloatValues) > 0 ||
-		len(k.mergedIntegerValues) > 0 ||
-		len(k.mergedUnsignedValues) > 0 ||
-		len(k.mergedStringValues) > 0 ||
-		len(k.mergedBooleanValues) > 0
-}
-
-func (k *tsmKeyIterator) EstimatedIndexSize() int {
-	var size uint32
-	for _, r := range k.readers {
-		size += r.IndexSize()
-	}
-	return int(size) / len(k.readers)
-}
-
-// Next returns true if there are any values remaining in the iterator.
-func (k *tsmKeyIterator) Next() bool {
-RETRY:
-	// Any merged blocks pending?
-	if len(k.merged) > 0 {
-		k.merged = k.merged[1:]
-		if len(k.merged) > 0 {
-			return true
-		}
-	}
-
-	// Any merged values pending?
-	if k.hasMergedValues() {
-		k.merge()
-		if len(k.merged) > 0 || k.hasMergedValues() {
-			return true
-		}
-	}
-
-	// If we still have blocks from the last read, merge them
-	if len(k.blocks) > 0 {
-		k.merge()
-		if len(k.merged) > 0 || k.hasMergedValues() {
-			return true
-		}
-	}
-
-	// Read the next block from each TSM iterator
-	for i, v := range k.buf {
-		if len(v) == 0 {
-			iter := k.iterators[i]
-			if iter.Next() {
-				key, minTime, maxTime, typ, _, b, err := iter.Read()
-				if err != nil {
-					k.AppendError(err)
-				}
-
-				// This block may have ranges of time removed from it that would
-				// reduce the block min and max time.
-				tombstones := iter.r.TombstoneRange(key)
-
-				var blk *block
-				if cap(k.buf[i]) > len(k.buf[i]) {
-					k.buf[i] = k.buf[i][:len(k.buf[i])+1]
-					blk = k.buf[i][len(k.buf[i])-1]
-					if blk == nil {
-						blk = &block{}
-						k.buf[i][len(k.buf[i])-1] = blk
-					}
-				} else {
-					blk = &block{}
-					k.buf[i] = append(k.buf[i], blk)
-				}
-				blk.minTime = minTime
-				blk.maxTime = maxTime
-				blk.key = key
-				blk.typ = typ
-				blk.b = b
-				blk.tombstones = tombstones
-				blk.readMin = math.MaxInt64
-				blk.readMax = math.MinInt64
-
-				blockKey := key
-				for bytes.Equal(iter.PeekNext(), blockKey) {
-					iter.Next()
-					key, minTime, maxTime, typ, _, b, err := iter.Read()
-					if err != nil {
-						k.AppendError(err)
-					}
-
-					tombstones := iter.r.TombstoneRange(key)
-
-					var blk *block
-					if cap(k.buf[i]) > len(k.buf[i]) {
-						k.buf[i] = k.buf[i][:len(k.buf[i])+1]
-						blk = k.buf[i][len(k.buf[i])-1]
-						if blk == nil {
-							blk = &block{}
-							k.buf[i][len(k.buf[i])-1] = blk
-						}
-					} else {
-						blk = &block{}
-						k.buf[i] = append(k.buf[i], blk)
-					}
-
-					blk.minTime = minTime
-					blk.maxTime = maxTime
-					blk.key = key
-					blk.typ = typ
-					blk.b = b
-					blk.tombstones = tombstones
-					blk.readMin = math.MaxInt64
-					blk.readMax = math.MinInt64
-				}
-			}
-
-			if iter.Err() != nil {
-				k.AppendError(iter.Err())
-			}
-		}
-	}
-
-	// Each reader could have a different key that it's currently at, need to find
-	// the next smallest one to keep the sort ordering.
-	var minKey []byte
-	var minType byte
-	for _, b := range k.buf {
-		// block could be nil if the iterator has been exhausted for that file
-		if len(b) == 0 {
-			continue
-		}
-		if len(minKey) == 0 || bytes.Compare(b[0].key, minKey) < 0 {
-			minKey = b[0].key
-			minType = b[0].typ
-		}
-	}
-	k.key = minKey
-	k.typ = minType
-
-	// Now we need to find all blocks that match the min key so we can combine and dedupe
-	// the blocks if necessary
-	for i, b := range k.buf {
-		if len(b) == 0 {
-			continue
-		}
-		if bytes.Equal(b[0].key, k.key) {
-			k.blocks = append(k.blocks, b...)
-			k.buf[i] = k.buf[i][:0]
-		}
-	}
-
-	if len(k.blocks) == 0 {
-		return false
-	}
-
-	k.merge()
-
-	// After merging all the values for this key, we might not have any.  (e.g. they were all deleted
-	// through many tombstones).  In this case, move on to the next key instead of ending iteration.
-	if len(k.merged) == 0 {
-		goto RETRY
-	}
-
-	return len(k.merged) > 0
-}
-
-// merge combines the next set of blocks into merged blocks.
-func (k *tsmKeyIterator) merge() {
-	switch k.typ {
-	case BlockFloat64:
-		k.mergeFloat()
-	case BlockInteger:
-		k.mergeInteger()
-	case BlockUnsigned:
-		k.mergeUnsigned()
-	case BlockBoolean:
-		k.mergeBoolean()
-	case BlockString:
-		k.mergeString()
-	default:
-		k.AppendError(fmt.Errorf("unknown block type: %v", k.typ))
-	}
-}
-
-func (k *tsmKeyIterator) Read() ([]byte, int64, int64, []byte, error) {
-	// See if compactions were disabled while we were running.
-	select {
-	case <-k.interrupt:
-		return nil, 0, 0, nil, errCompactionAborted{}
-	default:
-	}
-
-	if len(k.merged) == 0 {
-		return nil, 0, 0, nil, k.Err()
-	}
-
-	block := k.merged[0]
-	return block.key, block.minTime, block.maxTime, block.b, k.Err()
-}
-
-func (k *tsmKeyIterator) Close() error {
-	k.values = nil
-	k.pos = nil
-	k.iterators = nil
-	for _, r := range k.readers {
-		if err := r.Close(); err != nil {
-			return err
-		}
-	}
-	return nil
-}
-
-// Error returns any errors encountered during iteration.
-func (k *tsmKeyIterator) Err() error {
-	if len(k.errs) == 0 {
-		return nil
-	}
-	return k.errs
-}
-
 // tsmBatchKeyIterator implements the KeyIterator for set of TSMReaders.  Iteration produces
 // keys in sorted order and the values between the keys sorted and deduped.  If any of
 // the readers have associated tombstone entries, they are returned as part of iteration.
@@ -1644,15 +1377,28 @@ type tsmBatchKeyIterator struct {
 	// without decode
 	merged    blocks
 	interrupt chan struct{}
+
+	// maxErrors is the maximum number of errors to store before discarding.
+	maxErrors int
+	// overflowErrors is the number of errors we have ignored.
+	overflowErrors int
 }
 
-func (t *tsmBatchKeyIterator) AppendError(err error) {
-	t.errs = append(t.errs, err)
+func (t *tsmBatchKeyIterator) AppendError(err error) bool {
+	if t.maxErrors > len(t.errs) {
+		t.errs = append(t.errs, err)
+		// Was the error stored?
+		return true
+	} else {
+		// Was the error dropped
+		t.overflowErrors++
+		return false
+	}
 }
 
 // NewTSMBatchKeyIterator returns a new TSM key iterator from readers.
 // size indicates the maximum number of values to encode in a single block.
-func NewTSMBatchKeyIterator(size int, fast bool, interrupt chan struct{}, tsmFiles []string, readers ...*TSMReader) (KeyIterator, error) {
+func NewTSMBatchKeyIterator(size int, fast bool, maxErrors int, interrupt chan struct{}, tsmFiles []string, readers ...*TSMReader) (KeyIterator, error) {
 	var iter []*BlockIterator
 	for _, r := range readers {
 		iter = append(iter, r.BlockIterator())
@@ -1673,6 +1419,7 @@ func NewTSMBatchKeyIterator(size int, fast bool, interrupt chan struct{}, tsmFil
 		mergedBooleanValues:  &tsdb.BooleanArray{},
 		mergedStringValues:   &tsdb.StringArray{},
 		interrupt:            interrupt,
+		maxErrors:            maxErrors,
 	}, nil
 }
 
@@ -1887,20 +1634,24 @@ func (k *tsmBatchKeyIterator) Close() error {
 	k.values = nil
 	k.pos = nil
 	k.iterators = nil
+	var errSlice []error
 	for _, r := range k.readers {
-		if err := r.Close(); err != nil {
-			return err
-		}
+		errSlice = append(errSlice, r.Close())
 	}
-	return nil
+	return errors.Join(errSlice...)
 }
 
-// Error returns any errors encountered during iteration.
+// Err error returns any errors encountered during iteration.
 func (k *tsmBatchKeyIterator) Err() error {
 	if len(k.errs) == 0 {
 		return nil
 	}
-	return k.errs
+	// Copy the errors before appending the dropped error count
+	var errs TSMErrors
+	errs = make([]error, 0, len(k.errs)+1)
+	errs = append(errs, k.errs...)
+	errs = append(errs, fmt.Errorf("additional errors dropped: %d", k.overflowErrors))
+	return errs
 }
 
 type cacheKeyIterator struct {
