@@ -2,11 +2,11 @@ package tsm1
 
 import (
 	"fmt"
-	"io/ioutil"
 	"math"
 	"os"
 	"path/filepath"
 	"sort"
+	"strings"
 	"testing"
 
 	"github.com/stretchr/testify/require"
@@ -17,8 +17,7 @@ func fatal(t *testing.T, msg string, err error) {
 }
 
 func TestTSMReader_Type(t *testing.T) {
-	dir := mustTempDir()
-	defer os.RemoveAll(dir)
+	dir := t.TempDir()
 	f := mustTempFile(dir)
 
 	w, err := NewTSMWriter(f)
@@ -47,6 +46,7 @@ func TestTSMReader_Type(t *testing.T) {
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
+	t.Cleanup(func() { r.Close() })
 
 	typ, err := r.Type([]byte("cpu"))
 	if err != nil {
@@ -59,8 +59,7 @@ func TestTSMReader_Type(t *testing.T) {
 }
 
 func TestTSMReader_MMAP_ReadAll(t *testing.T) {
-	dir := mustTempDir()
-	defer os.RemoveAll(dir)
+	dir := t.TempDir()
 	f := mustTempFile(dir)
 	defer f.Close()
 
@@ -70,11 +69,11 @@ func TestTSMReader_MMAP_ReadAll(t *testing.T) {
 	}
 
 	var data = map[string][]Value{
-		"float":  []Value{NewValue(1, 1.0)},
-		"int":    []Value{NewValue(1, int64(1))},
-		"uint":   []Value{NewValue(1, ^uint64(0))},
-		"bool":   []Value{NewValue(1, true)},
-		"string": []Value{NewValue(1, "foo")},
+		"float":  {NewValue(1, 1.0)},
+		"int":    {NewValue(1, int64(1))},
+		"uint":   {NewValue(1, ^uint64(0))},
+		"bool":   {NewValue(1, true)},
+		"string": {NewValue(1, "foo")},
 	}
 
 	keys := make([]string, 0, len(data))
@@ -133,8 +132,7 @@ func TestTSMReader_MMAP_ReadAll(t *testing.T) {
 }
 
 func TestTSMReader_MMAP_Read(t *testing.T) {
-	dir := mustTempDir()
-	defer os.RemoveAll(dir)
+	dir := t.TempDir()
 	f := mustTempFile(dir)
 	defer f.Close()
 
@@ -144,15 +142,15 @@ func TestTSMReader_MMAP_Read(t *testing.T) {
 	}
 
 	var data = map[string][]Value{
-		"float": []Value{
+		"float": {
 			NewValue(1, 1.0)},
-		"int": []Value{
+		"int": {
 			NewValue(1, int64(1))},
-		"uint": []Value{
+		"uint": {
 			NewValue(1, ^uint64(0))},
-		"bool": []Value{
+		"bool": {
 			NewValue(1, true)},
-		"string": []Value{
+		"string": {
 			NewValue(1, "foo")},
 	}
 
@@ -212,8 +210,7 @@ func TestTSMReader_MMAP_Read(t *testing.T) {
 }
 
 func TestTSMReader_MMAP_Keys(t *testing.T) {
-	dir := mustTempDir()
-	defer os.RemoveAll(dir)
+	dir := t.TempDir()
 	f := mustTempFile(dir)
 	defer f.Close()
 
@@ -223,15 +220,15 @@ func TestTSMReader_MMAP_Keys(t *testing.T) {
 	}
 
 	var data = map[string][]Value{
-		"float": []Value{
+		"float": {
 			NewValue(1, 1.0)},
-		"int": []Value{
+		"int": {
 			NewValue(1, int64(1))},
-		"uint": []Value{
+		"uint": {
 			NewValue(1, ^uint64(0))},
-		"bool": []Value{
+		"bool": {
 			NewValue(1, true)},
-		"string": []Value{
+		"string": {
 			NewValue(1, "foo")},
 	}
 
@@ -291,10 +288,9 @@ func TestTSMReader_MMAP_Keys(t *testing.T) {
 }
 
 func TestTSMReader_MMAP_Tombstone(t *testing.T) {
-	dir := mustTempDir()
-	defer os.RemoveAll(dir)
+	dir := t.TempDir()
 	f := mustTempFile(dir)
-	defer f.Close()
+	t.Cleanup(func() { f.Close() })
 
 	w, err := NewTSMWriter(f)
 	if err != nil {
@@ -323,29 +319,29 @@ func TestTSMReader_MMAP_Tombstone(t *testing.T) {
 		t.Fatalf("unexpected error open file: %v", err)
 	}
 
-	r, err := NewTSMReader(f)
+	r1, err := NewTSMReader(f)
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
+	t.Cleanup(func() { r1.Close() })
 
-	if err := r.Delete([][]byte{[]byte("mem")}); err != nil {
+	if err := r1.Delete([][]byte{[]byte("mem")}); err != nil {
 		t.Fatalf("unexpected error deleting: %v", err)
 	}
 
-	r, err = NewTSMReader(f)
+	r2, err := NewTSMReader(f)
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
-	defer r.Close()
+	t.Cleanup(func() { r2.Close() })
 
-	if got, exp := r.KeyCount(), 1; got != exp {
+	if got, exp := r2.KeyCount(), 1; got != exp {
 		t.Fatalf("key length mismatch: got %v, exp %v", got, exp)
 	}
 }
 
 func TestTSMReader_MMAP_TombstoneRange(t *testing.T) {
-	dir := mustTempDir()
-	defer os.RemoveAll(dir)
+	dir := t.TempDir()
 	f := mustTempFile(dir)
 	defer f.Close()
 
@@ -409,8 +405,7 @@ func TestTSMReader_MMAP_TombstoneRange(t *testing.T) {
 }
 
 func TestTSMReader_MMAP_TombstoneOutsideTimeRange(t *testing.T) {
-	dir := mustTempDir()
-	defer os.RemoveAll(dir)
+	dir := t.TempDir()
 	f := mustTempFile(dir)
 	defer f.Close()
 
@@ -471,8 +466,7 @@ func TestTSMReader_MMAP_TombstoneOutsideTimeRange(t *testing.T) {
 }
 
 func TestTSMReader_MMAP_TombstoneOutsideKeyRange(t *testing.T) {
-	dir := mustTempDir()
-	defer os.RemoveAll(dir)
+	dir := t.TempDir()
 	f := mustTempFile(dir)
 	defer f.Close()
 
@@ -533,8 +527,7 @@ func TestTSMReader_MMAP_TombstoneOutsideKeyRange(t *testing.T) {
 }
 
 func TestTSMReader_MMAP_TombstoneOverlapKeyRange(t *testing.T) {
-	dir := mustTempDir()
-	defer os.RemoveAll(dir)
+	dir := t.TempDir()
 	f := mustTempFile(dir)
 	defer f.Close()
 
@@ -599,8 +592,7 @@ func TestTSMReader_MMAP_TombstoneOverlapKeyRange(t *testing.T) {
 }
 
 func TestTSMReader_MMAP_TombstoneFullRange(t *testing.T) {
-	dir := mustTempDir()
-	defer os.RemoveAll(dir)
+	dir := t.TempDir()
 	f := mustTempFile(dir)
 	defer f.Close()
 
@@ -652,8 +644,7 @@ func TestTSMReader_MMAP_TombstoneFullRange(t *testing.T) {
 }
 
 func TestTSMReader_MMAP_TombstoneFullRangeMultiple(t *testing.T) {
-	dir := mustTempDir()
-	defer os.RemoveAll(dir)
+	dir := t.TempDir()
 	f := mustTempFile(dir)
 	defer f.Close()
 
@@ -725,8 +716,7 @@ func TestTSMReader_MMAP_TombstoneFullRangeMultiple(t *testing.T) {
 }
 
 func TestTSMReader_MMAP_TombstoneMultipleRanges(t *testing.T) {
-	dir := mustTempDir()
-	defer os.RemoveAll(dir)
+	dir := t.TempDir()
 	f := mustTempFile(dir)
 	defer f.Close()
 
@@ -784,8 +774,7 @@ func TestTSMReader_MMAP_TombstoneMultipleRanges(t *testing.T) {
 }
 
 func TestTSMReader_MMAP_TombstoneMultipleRangesFull(t *testing.T) {
-	dir := mustTempDir()
-	defer os.RemoveAll(dir)
+	dir := t.TempDir()
 	f := mustTempFile(dir)
 	defer f.Close()
 
@@ -844,8 +833,7 @@ func TestTSMReader_MMAP_TombstoneMultipleRangesFull(t *testing.T) {
 }
 
 func TestTSMReader_MMAP_TombstoneMultipleRangesNoOverlap(t *testing.T) {
-	dir := mustTempDir()
-	defer os.RemoveAll(dir)
+	dir := t.TempDir()
 	f := mustTempFile(dir)
 	defer f.Close()
 
@@ -905,8 +893,7 @@ func TestTSMReader_MMAP_TombstoneMultipleRangesNoOverlap(t *testing.T) {
 }
 
 func TestTSMReader_MMAP_TombstoneOutsideRange(t *testing.T) {
-	dir := mustTempDir()
-	defer os.RemoveAll(dir)
+	dir := t.TempDir()
 	f := mustTempFile(dir)
 	defer f.Close()
 
@@ -989,8 +976,7 @@ func TestTSMReader_MMAP_TombstoneOutsideRange(t *testing.T) {
 }
 
 func TestTSMReader_MMAP_Stats(t *testing.T) {
-	dir := mustTempDir()
-	defer os.RemoveAll(dir)
+	dir := t.TempDir()
 	f := mustTempFile(dir)
 	defer f.Close()
 
@@ -1052,8 +1038,7 @@ func TestTSMReader_MMAP_Stats(t *testing.T) {
 
 // Ensure that we return an error if we try to open a non-tsm file
 func TestTSMReader_VerifiesFileType(t *testing.T) {
-	dir := mustTempDir()
-	defer os.RemoveAll(dir)
+	dir := t.TempDir()
 	f := mustTempFile(dir)
 	defer f.Close()
 
@@ -1081,7 +1066,7 @@ func TestIndirectIndex_Entries(t *testing.T) {
 
 	indirect := NewIndirectIndex()
 	if err := indirect.UnmarshalBinary(b); err != nil {
-		t.Fatalf("unexpected error unmarshaling index: %v", err)
+		t.Fatalf("unexpected error unmarshalling index: %v", err)
 	}
 
 	entries := indirect.Entries([]byte("cpu"))
@@ -1121,7 +1106,7 @@ func TestIndirectIndex_Entries_NonExistent(t *testing.T) {
 
 	indirect := NewIndirectIndex()
 	if err := indirect.UnmarshalBinary(b); err != nil {
-		t.Fatalf("unexpected error unmarshaling index: %v", err)
+		t.Fatalf("unexpected error unmarshalling index: %v", err)
 	}
 
 	// mem has not been added to the index so we should get no entries back
@@ -1184,8 +1169,7 @@ func TestDirectIndex_KeyCount(t *testing.T) {
 }
 
 func TestBlockIterator_Single(t *testing.T) {
-	dir := mustTempDir()
-	defer os.RemoveAll(dir)
+	dir := t.TempDir()
 	f := mustTempFile(dir)
 
 	w, err := NewTSMWriter(f)
@@ -1215,6 +1199,7 @@ func TestBlockIterator_Single(t *testing.T) {
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
+	t.Cleanup(func() { r.Close() })
 
 	var count int
 	iter := r.BlockIterator()
@@ -1253,8 +1238,7 @@ func TestBlockIterator_Single(t *testing.T) {
 }
 
 func TestBlockIterator_Tombstone(t *testing.T) {
-	dir := mustTempDir()
-	defer os.RemoveAll(dir)
+	dir := t.TempDir()
 	f := mustTempFile(dir)
 
 	w, err := NewTSMWriter(f)
@@ -1288,6 +1272,7 @@ func TestBlockIterator_Tombstone(t *testing.T) {
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
+	t.Cleanup(func() { r.Close() })
 
 	iter := r.BlockIterator()
 	for iter.Next() {
@@ -1302,8 +1287,7 @@ func TestBlockIterator_Tombstone(t *testing.T) {
 }
 
 func TestBlockIterator_MultipleBlocks(t *testing.T) {
-	dir := mustTempDir()
-	defer os.RemoveAll(dir)
+	dir := t.TempDir()
 	f := mustTempFile(dir)
 
 	w, err := NewTSMWriter(f)
@@ -1338,6 +1322,7 @@ func TestBlockIterator_MultipleBlocks(t *testing.T) {
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
+	t.Cleanup(func() { r.Close() })
 
 	var count int
 	expData := []Values{values1, values2}
@@ -1380,8 +1365,7 @@ func TestBlockIterator_MultipleBlocks(t *testing.T) {
 }
 
 func TestBlockIterator_Sorted(t *testing.T) {
-	dir := mustTempDir()
-	defer os.RemoveAll(dir)
+	dir := t.TempDir()
 	f := mustTempFile(dir)
 
 	w, err := NewTSMWriter(f)
@@ -1390,11 +1374,11 @@ func TestBlockIterator_Sorted(t *testing.T) {
 	}
 
 	values := map[string][]Value{
-		"mem":    []Value{NewValue(0, int64(1))},
-		"cycles": []Value{NewValue(0, ^uint64(0))},
-		"cpu":    []Value{NewValue(1, float64(2))},
-		"disk":   []Value{NewValue(1, true)},
-		"load":   []Value{NewValue(1, "string")},
+		"mem":    {NewValue(0, int64(1))},
+		"cycles": {NewValue(0, ^uint64(0))},
+		"cpu":    {NewValue(1, float64(2))},
+		"disk":   {NewValue(1, true)},
+		"load":   {NewValue(1, "string")},
 	}
 
 	keys := make([]string, 0, len(values))
@@ -1406,7 +1390,6 @@ func TestBlockIterator_Sorted(t *testing.T) {
 	for _, k := range keys {
 		if err := w.Write([]byte(k), values[k]); err != nil {
 			t.Fatalf("unexpected error writing: %v", err)
-
 		}
 	}
 
@@ -1427,6 +1410,7 @@ func TestBlockIterator_Sorted(t *testing.T) {
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
+	t.Cleanup(func() { r.Close() })
 
 	var count int
 	iter := r.BlockIterator()
@@ -1457,8 +1441,7 @@ func TestBlockIterator_Sorted(t *testing.T) {
 }
 
 func TestIndirectIndex_UnmarshalBinary_BlockCountOverflow(t *testing.T) {
-	dir := mustTempDir()
-	defer os.RemoveAll(dir)
+	dir := t.TempDir()
 	f := mustTempFile(dir)
 	defer f.Close()
 
@@ -1492,8 +1475,7 @@ func TestIndirectIndex_UnmarshalBinary_BlockCountOverflow(t *testing.T) {
 }
 
 func TestCompacted_NotFull(t *testing.T) {
-	dir := mustTempDir()
-	defer os.RemoveAll(dir)
+	dir := t.TempDir()
 	f := mustTempFile(dir)
 
 	w, err := NewTSMWriter(f)
@@ -1523,6 +1505,7 @@ func TestCompacted_NotFull(t *testing.T) {
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
+	t.Cleanup(func() { r.Close() })
 
 	iter := r.BlockIterator()
 	if !iter.Next() {
@@ -1544,8 +1527,7 @@ func TestCompacted_NotFull(t *testing.T) {
 }
 
 func TestTSMReader_File_ReadAll(t *testing.T) {
-	dir := mustTempDir()
-	defer os.RemoveAll(dir)
+	dir := t.TempDir()
 	f := mustTempFile(dir)
 	defer f.Close()
 
@@ -1555,15 +1537,15 @@ func TestTSMReader_File_ReadAll(t *testing.T) {
 	}
 
 	var data = map[string][]Value{
-		"float": []Value{
+		"float": {
 			NewValue(1, 1.0)},
-		"int": []Value{
+		"int": {
 			NewValue(1, int64(1))},
-		"uint": []Value{
+		"uint": {
 			NewValue(1, ^uint64(0))},
-		"bool": []Value{
+		"bool": {
 			NewValue(1, true)},
-		"string": []Value{
+		"string": {
 			NewValue(1, "foo")},
 	}
 
@@ -1652,48 +1634,43 @@ func TestTSMReader_FuzzCrashes(t *testing.T) {
 	}
 
 	for _, c := range cases {
-		func() {
-			dir := mustTempDir()
-			defer os.RemoveAll(dir)
+		dir := t.TempDir()
 
-			filename := filepath.Join(dir, "x.tsm")
-			if err := ioutil.WriteFile(filename, []byte(c), 0600); err != nil {
-				t.Fatalf("exp no error, got %s", err)
-			}
-			defer os.RemoveAll(dir)
+		filename := filepath.Join(dir, "x.tsm")
+		if err := os.WriteFile(filename, []byte(c), 0600); err != nil {
+			t.Fatalf("exp no error, got %s", err)
+		}
 
-			f, err := os.Open(filename)
-			if err != nil {
-				t.Fatalf("exp no error, got %s", err)
-			}
-			defer f.Close()
+		f, err := os.Open(filename)
+		if err != nil {
+			t.Fatalf("exp no error, got %s", err)
+		}
+		t.Cleanup(func() { f.Close() })
 
-			r, err := NewTSMReader(f)
+		r, err := NewTSMReader(f)
+		if err != nil {
+			return
+		}
+		t.Cleanup(func() { r.Close() })
+
+		iter := r.BlockIterator()
+		for iter.Next() {
+			key, _, _, _, _, _, err := iter.Read()
 			if err != nil {
 				return
 			}
-			defer r.Close()
 
-			iter := r.BlockIterator()
-			for iter.Next() {
-				key, _, _, _, _, _, err := iter.Read()
-				if err != nil {
-					return
-				}
+			_, _ = r.Type(key)
 
-				_, _ = r.Type(key)
-
-				if _, err = r.ReadAll(key); err != nil {
-					return
-				}
+			if _, err = r.ReadAll(key); err != nil {
+				return
 			}
-		}()
+		}
 	}
 }
 
 func TestTSMReader_File_Read(t *testing.T) {
-	dir := mustTempDir()
-	defer os.RemoveAll(dir)
+	dir := t.TempDir()
 	f := mustTempFile(dir)
 	defer f.Close()
 
@@ -1703,15 +1680,15 @@ func TestTSMReader_File_Read(t *testing.T) {
 	}
 
 	var data = map[string][]Value{
-		"float": []Value{
+		"float": {
 			NewValue(1, 1.0)},
-		"int": []Value{
+		"int": {
 			NewValue(1, int64(1))},
-		"uint": []Value{
+		"uint": {
 			NewValue(1, ^uint64(0))},
-		"bool": []Value{
+		"bool": {
 			NewValue(1, true)},
-		"string": []Value{
+		"string": {
 			NewValue(1, "foo")},
 	}
 
@@ -1771,8 +1748,7 @@ func TestTSMReader_File_Read(t *testing.T) {
 }
 
 func TestTSMReader_References(t *testing.T) {
-	dir := mustTempDir()
-	defer os.RemoveAll(dir)
+	dir := t.TempDir()
 	f := mustTempFile(dir)
 	defer f.Close()
 
@@ -1782,15 +1758,15 @@ func TestTSMReader_References(t *testing.T) {
 	}
 
 	var data = map[string][]Value{
-		"float": []Value{
+		"float": {
 			NewValue(1, 1.0)},
-		"int": []Value{
+		"int": {
 			NewValue(1, int64(1))},
-		"uint": []Value{
+		"uint": {
 			NewValue(1, ^uint64(0))},
-		"bool": []Value{
+		"bool": {
 			NewValue(1, true)},
-		"string": []Value{
+		"string": {
 			NewValue(1, "foo")},
 	}
 
@@ -1864,6 +1840,98 @@ func TestTSMReader_References(t *testing.T) {
 	}
 }
 
+func TestBatchKeyIterator_Errors(t *testing.T) {
+	const MaxErrors = 10
+
+	dir, name := createTestTSM(t)
+	defer os.RemoveAll(dir)
+	fr, err := os.Open(name)
+	if err != nil {
+		t.Fatalf("unexpected error opening file %s: %v", name, err)
+	}
+	r, err := NewTSMReader(fr)
+	if err != nil {
+		// Only have a deferred close if we could not create the TSMReader
+		defer func() {
+			if e := fr.Close(); e != nil {
+				t.Fatalf("unexpected error closing %s: %v", name, e)
+			}
+		}()
+
+		t.Fatalf("unexpected error creating TSMReader for %s: %v", name, err)
+	}
+	defer func() {
+		if e := r.Close(); e != nil {
+			t.Fatalf("error closing TSMReader for %s: %v", name, e)
+		}
+	}()
+	interrupts := make(chan struct{})
+	var iter KeyIterator
+	if iter, err = NewTSMBatchKeyIterator(3, false, MaxErrors, interrupts, []string{name}, r); err != nil {
+		t.Fatalf("unexpected error creating tsmBatchKeyIterator: %v", err)
+	}
+	var i int
+	for i = 0; i < MaxErrors*2; i++ {
+		saved := iter.(*tsmBatchKeyIterator).AppendError(fmt.Errorf("fake error: %d", i))
+		if i < MaxErrors && !saved {
+			t.Fatalf("error unexpectedly not saved: %d", i)
+		}
+		if i >= MaxErrors && saved {
+			t.Fatalf("error unexpectedly saved: %d", i)
+		}
+	}
+	errs := iter.Err()
+	if errCnt := len(errs.(TSMErrors)); errCnt != (MaxErrors + 1) {
+		t.Fatalf("saved wrong number of errors: expected %d, got %d", MaxErrors, errCnt)
+	}
+	expected := fmt.Sprintf("additional errors dropped: %d", i-MaxErrors)
+	if strings.Compare(errs.(TSMErrors)[MaxErrors].Error(), expected) != 0 {
+		t.Fatalf("expected: '%s', got: '%s", expected, errs.(TSMErrors)[MaxErrors].Error())
+	}
+}
+
+func createTestTSM(t *testing.T) (dir string, name string) {
+	dir = t.TempDir()
+	f := mustTempFile(dir)
+	name = f.Name()
+	w, err := NewTSMWriter(f)
+	if err != nil {
+		f.Close()
+		t.Fatalf("unexpected error creating writer for %s: %v", name, err)
+	}
+	defer func() {
+		if e := w.Close(); e != nil {
+			t.Fatalf("write TSM close of %s: %v", name, err)
+		}
+	}()
+
+	var data = map[string][]Value{
+		"float":  {NewValue(1, 1.0)},
+		"int":    {NewValue(1, int64(1))},
+		"uint":   {NewValue(1, ^uint64(0))},
+		"bool":   {NewValue(1, true)},
+		"string": {NewValue(1, "foo")},
+	}
+
+	keys := make([]string, 0, len(data))
+	for k := range data {
+		keys = append(keys, k)
+	}
+	sort.Strings(keys)
+
+	for _, k := range keys {
+		if err := w.Write([]byte(k), data[k]); err != nil {
+			t.Fatalf("write TSM value: %v", err)
+		}
+	}
+
+	if err := w.WriteIndex(); err != nil {
+		t.Fatalf("write TSM index: %v", err)
+	}
+
+	return dir, name
+}
+
 func BenchmarkIndirectIndex_UnmarshalBinary(b *testing.B) {
 	index := NewIndexWriter()
 	for i := 0; i < 100000; i++ {
@@ -1879,7 +1947,7 @@ func BenchmarkIndirectIndex_UnmarshalBinary(b *testing.B) {
 	b.ResetTimer()
 	for i := 0; i < b.N; i++ {
 		if err := indirect.UnmarshalBinary(bytes); err != nil {
-			b.Fatalf("unexpected error unmarshaling index: %v", err)
+			b.Fatalf("unexpected error unmarshalling index: %v", err)
 		}
 	}
 }
@@ -1900,7 +1968,7 @@ func mustMakeIndex(tb testing.TB, keys, blocks int) *indirectIndex {
 
 	indirect := NewIndirectIndex()
 	if err = indirect.UnmarshalBinary(bytes); err != nil {
-		tb.Fatalf("unexpected error unmarshaling index: %v", err)
+		tb.Fatalf("unexpected error unmarshalling index: %v", err)
 	}
 
 	return indirect
