@@ -2,19 +2,17 @@ package tsm1_test
 
 import (
 	"bytes"
-	"io/ioutil"
 	"os"
 	"testing"
 
-	"github.com/influxdata/influxdb/tsdb/engine/tsm1"
+	"github.com/influxdata/influxdb/v2/tsdb/engine/tsm1"
 	"github.com/stretchr/testify/require"
 )
 
 func TestTombstoner_Add(t *testing.T) {
-	dir := MustTempDir()
-	defer func() { os.RemoveAll(dir) }()
+	dir := t.TempDir()
 
-	f := MustTempFile(dir)
+	f := MustTempFile(t, dir)
 	ts := tsm1.NewTombstoner(f.Name(), nil)
 
 	entries := mustReadAll(ts)
@@ -59,10 +57,9 @@ func TestTombstoner_Add(t *testing.T) {
 }
 
 func TestTombstoner_Add_LargeKey(t *testing.T) {
-	dir := MustTempDir()
-	defer func() { os.RemoveAll(dir) }()
+	dir := t.TempDir()
 
-	f := MustTempFile(dir)
+	f := MustTempFile(t, dir)
 	ts := tsm1.NewTombstoner(f.Name(), nil)
 
 	entries := mustReadAll(ts)
@@ -108,10 +105,9 @@ func TestTombstoner_Add_LargeKey(t *testing.T) {
 }
 
 func TestTombstoner_Add_Multiple(t *testing.T) {
-	dir := MustTempDir()
-	defer func() { os.RemoveAll(dir) }()
+	dir := t.TempDir()
 
-	f := MustTempFile(dir)
+	f := MustTempFile(t, dir)
 	ts := tsm1.NewTombstoner(f.Name(), nil)
 
 	entries := mustReadAll(ts)
@@ -171,10 +167,9 @@ func TestTombstoner_Add_Multiple(t *testing.T) {
 }
 
 func TestTombstoner_Add_Empty(t *testing.T) {
-	dir := MustTempDir()
-	defer func() { os.RemoveAll(dir) }()
+	dir := t.TempDir()
 
-	f := MustTempFile(dir)
+	f := MustTempFile(t, dir)
 	ts := tsm1.NewTombstoner(f.Name(), nil)
 
 	entries := mustReadAll(ts)
@@ -200,10 +195,9 @@ func TestTombstoner_Add_Empty(t *testing.T) {
 }
 
 func TestTombstoner_Delete(t *testing.T) {
-	dir := MustTempDir()
-	defer func() { os.RemoveAll(dir) }()
+	dir := t.TempDir()
 
-	f := MustTempFile(dir)
+	f := MustTempFile(t, dir)
 	ts := tsm1.NewTombstoner(f.Name(), nil)
 
 	ts.Add([][]byte{[]byte("foo")})
@@ -238,11 +232,10 @@ func TestTombstoner_Delete(t *testing.T) {
 }
 
 func TestTombstoner_ReadV1(t *testing.T) {
-	dir := MustTempDir()
-	defer func() { os.RemoveAll(dir) }()
+	dir := t.TempDir()
 
-	f := MustTempFile(dir)
-	if err := ioutil.WriteFile(f.Name(), []byte("foo\n"), 0x0600); err != nil {
+	f := MustTempFile(t, dir)
+	if err := os.WriteFile(f.Name(), []byte("foo\n"), 0x0600); err != nil {
 		t.Fatalf("write v1 file: %v", err)
 	}
 	f.Close()
@@ -280,12 +273,10 @@ func TestTombstoner_ReadV1(t *testing.T) {
 }
 
 func TestTombstoner_ReadEmptyV1(t *testing.T) {
-	dir := MustTempDir()
-	defer func() { os.RemoveAll(dir) }()
+	dir := t.TempDir()
 
-	f := MustTempFile(dir)
+	f := MustTempFile(t, dir)
 	f.Close()
-
 	if err := os.Rename(f.Name(), f.Name()+"."+tsm1.TombstoneFileExtension); err != nil {
 		t.Fatalf("rename tombstone failed: %v", err)
 	}
