@@ -4,11 +4,10 @@ import (
 	"bytes"
 	"encoding/binary"
 	"io"
-	"io/ioutil"
 	"os"
 	"testing"
 
-	"github.com/influxdata/influxdb/tsdb/engine/tsm1"
+	"github.com/influxdata/influxdb/v2/tsdb/engine/tsm1"
 )
 
 func TestTSMWriter_Write_Empty(t *testing.T) {
@@ -48,9 +47,8 @@ func TestTSMWriter_Write_NoValues(t *testing.T) {
 }
 
 func TestTSMWriter_Write_Single(t *testing.T) {
-	dir := MustTempDir()
-	defer os.RemoveAll(dir)
-	f := MustTempFile(dir)
+	dir := t.TempDir()
+	f := MustTempFile(t, dir)
 
 	w, err := tsm1.NewTSMWriter(f)
 	if err != nil {
@@ -75,7 +73,7 @@ func TestTSMWriter_Write_Single(t *testing.T) {
 		t.Fatalf("unexpected error open file: %v", err)
 	}
 
-	b, err := ioutil.ReadAll(fd)
+	b, err := io.ReadAll(fd)
 	if err != nil {
 		t.Fatalf("unexpected error reading: %v", err)
 	}
@@ -114,9 +112,8 @@ func TestTSMWriter_Write_Single(t *testing.T) {
 }
 
 func TestTSMWriter_Write_Multiple(t *testing.T) {
-	dir := MustTempDir()
-	defer os.RemoveAll(dir)
-	f := MustTempFile(dir)
+	dir := t.TempDir()
+	f := MustTempFile(t, dir)
 
 	w, err := tsm1.NewTSMWriter(f)
 	if err != nil {
@@ -175,9 +172,8 @@ func TestTSMWriter_Write_Multiple(t *testing.T) {
 }
 
 func TestTSMWriter_Write_MultipleKeyValues(t *testing.T) {
-	dir := MustTempDir()
-	defer os.RemoveAll(dir)
-	f := MustTempFile(dir)
+	dir := t.TempDir()
+	f := MustTempFile(t, dir)
 
 	w, err := tsm1.NewTSMWriter(f)
 	if err != nil {
@@ -243,9 +239,8 @@ func TestTSMWriter_Write_MultipleKeyValues(t *testing.T) {
 
 // Tests that writing keys in reverse is able to read them back.
 func TestTSMWriter_Write_SameKey(t *testing.T) {
-	dir := MustTempDir()
-	defer os.RemoveAll(dir)
-	f := MustTempFile(dir)
+	dir := t.TempDir()
+	f := MustTempFile(t, dir)
 
 	w, err := tsm1.NewTSMWriter(f)
 	if err != nil {
@@ -312,9 +307,8 @@ func TestTSMWriter_Write_SameKey(t *testing.T) {
 // Tests that calling Read returns all the values for block matching the key
 // and timestamp
 func TestTSMWriter_Read_Multiple(t *testing.T) {
-	dir := MustTempDir()
-	defer os.RemoveAll(dir)
-	f := MustTempFile(dir)
+	dir := t.TempDir()
+	f := MustTempFile(t, dir)
 
 	w, err := tsm1.NewTSMWriter(f)
 	if err != nil {
@@ -396,9 +390,8 @@ func TestTSMWriter_Read_Multiple(t *testing.T) {
 }
 
 func TestTSMWriter_WriteBlock_Empty(t *testing.T) {
-	dir := MustTempDir()
-	defer os.RemoveAll(dir)
-	f := MustTempFile(dir)
+	dir := t.TempDir()
+	f := MustTempFile(t, dir)
 
 	w, err := tsm1.NewTSMWriter(f)
 	if err != nil {
@@ -419,7 +412,7 @@ func TestTSMWriter_WriteBlock_Empty(t *testing.T) {
 	}
 	defer fd.Close()
 
-	b, err := ioutil.ReadAll(fd)
+	b, err := io.ReadAll(fd)
 	if err != nil {
 		t.Fatalf("unexpected error read all: %v", err)
 	}
@@ -430,9 +423,8 @@ func TestTSMWriter_WriteBlock_Empty(t *testing.T) {
 }
 
 func TestTSMWriter_WriteBlock_Multiple(t *testing.T) {
-	dir := MustTempDir()
-	defer os.RemoveAll(dir)
-	f := MustTempFile(dir)
+	dir := t.TempDir()
+	f := MustTempFile(t, dir)
 
 	w, err := tsm1.NewTSMWriter(f)
 	if err != nil {
@@ -467,7 +459,7 @@ func TestTSMWriter_WriteBlock_Multiple(t *testing.T) {
 	}
 	defer fd.Close()
 
-	b, err := ioutil.ReadAll(fd)
+	b, err := io.ReadAll(fd)
 	if err != nil {
 		t.Fatalf("unexpected error read all: %v", err)
 	}
@@ -488,8 +480,9 @@ func TestTSMWriter_WriteBlock_Multiple(t *testing.T) {
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
+	defer r.Close()
 
-	f = MustTempFile(dir)
+	f = MustTempFile(t, dir)
 	w, err = tsm1.NewTSMWriter(f)
 	if err != nil {
 		t.Fatalf("unexpected error creating writer: %v", err)
@@ -517,6 +510,7 @@ func TestTSMWriter_WriteBlock_Multiple(t *testing.T) {
 	if err != nil {
 		t.Fatalf("unexpected error open file: %v", err)
 	}
+	defer fd.Close()
 
 	// Now create a reader to verify the written blocks matches the originally
 	// written file using Write
@@ -545,9 +539,8 @@ func TestTSMWriter_WriteBlock_Multiple(t *testing.T) {
 }
 
 func TestTSMWriter_WriteBlock_MaxKey(t *testing.T) {
-	dir := MustTempDir()
-	defer os.RemoveAll(dir)
-	f := MustTempFile(dir)
+	dir := t.TempDir()
+	f := MustTempFile(t, dir)
 
 	w, err := tsm1.NewTSMWriter(f)
 	if err != nil {
@@ -565,10 +558,8 @@ func TestTSMWriter_WriteBlock_MaxKey(t *testing.T) {
 }
 
 func TestTSMWriter_Write_MaxKey(t *testing.T) {
-	dir := MustTempDir()
-	defer os.RemoveAll(dir)
-	f := MustTempFile(dir)
-	defer f.Close()
+	dir := t.TempDir()
+	f := MustTempFile(t, dir)
 
 	w, err := tsm1.NewTSMWriter(f)
 	if err != nil {
@@ -583,3 +574,41 @@ func TestTSMWriter_Write_MaxKey(t *testing.T) {
 		t.Fatalf("expected max key length error writing key: %v", err)
 	}
 }
+
+type fakeSyncer bool
+
+func (f *fakeSyncer) Sync() error {
+	*f = true
+	return nil
+}
+
+func TestTSMWriter_Sync(t *testing.T) {
+	f := &struct {
+		io.Writer
+		fakeSyncer
+	}{
+		Writer: io.Discard,
+	}
+
+	w, err := tsm1.NewTSMWriter(f)
+	if err != nil {
+		t.Fatalf("unexpected error creating writer: %v", err)
+	}
+
+	values := []tsm1.Value{tsm1.NewValue(0, 1.0)}
+	if err := w.Write([]byte("cpu"), values); err != nil {
+		t.Fatalf("unexpected error writing: %v", err)
+
+	}
+	if err := w.WriteIndex(); err != nil {
+		t.Fatalf("unexpected error writing index: %v", err)
+	}
+
+	if err := w.Close(); err != nil {
+		t.Fatalf("unexpected error closing: %v", err)
+	}
+
+	if !f.fakeSyncer {
+		t.Fatal("failed to sync")
+	}
+}
