@@ -2,19 +2,24 @@ package tsi1_test
 
 import (
 	"compress/gzip"
+	"errors"
 	"fmt"
-	"io/ioutil"
+	"io"
 	"os"
+	"path"
 	"path/filepath"
 	"reflect"
 	"regexp"
 	"sort"
 	"sync"
 	"testing"
+	"time"
 
-	"github.com/influxdata/influxdb/models"
-	"github.com/influxdata/influxdb/tsdb"
-	"github.com/influxdata/influxdb/tsdb/index/tsi1"
+	"github.com/influxdata/influxdb/v2/models"
+	"github.com/influxdata/influxdb/v2/tsdb"
+	"github.com/influxdata/influxdb/v2/tsdb/index/tsi1"
+	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
 )
 
 // Bloom filter settings used in tests.
@@ -22,7 +27,7 @@ const M, K = 4096, 6
 
 // Ensure index can iterate over all measurement names.
 func TestIndex_ForEachMeasurementName(t *testing.T) {
-	idx := MustOpenDefaultIndex()
+	idx := MustOpenDefaultIndex(t)
 	defer idx.Close()
 
 	// Add series to index.
@@ -75,7 +80,7 @@ func TestIndex_ForEachMeasurementName(t *testing.T) {
 
 // Ensure index can return whether a measurement exists.
 func TestIndex_MeasurementExists(t *testing.T) {
-	idx := MustOpenDefaultIndex()
+	idx := MustOpenDefaultIndex(t)
 	defer idx.Close()
 
 	// Add series to index.
@@ -137,7 +142,7 @@ func TestIndex_MeasurementExists(t *testing.T) {
 
 // Ensure index can return a list of matching measurements.
 func TestIndex_MeasurementNamesByRegex(t *testing.T) {
-	idx := MustOpenDefaultIndex()
+	idx := MustOpenDefaultIndex(t)
 	defer idx.Close()
 
 	// Add series to index.
@@ -162,7 +167,7 @@ func TestIndex_MeasurementNamesByRegex(t *testing.T) {
 
 // Ensure index can delete a measurement and all related keys, values, & series.
 func TestIndex_DropMeasurement(t *testing.T) {
-	idx := MustOpenDefaultIndex()
+	idx := MustOpenDefaultIndex(t)
 	defer idx.Close()
 
 	// Add series to index.
@@ -207,38 +212,78 @@ func TestIndex_DropMeasurement(t *testing.T) {
 	})
 }
 
+func TestIndex_OpenFail(t *testing.T) {
+	idx := NewDefaultIndex(t)
+	require.NoError(t, idx.Open())
+	idx.Index.Close()
+	// mess up the index:
+	tslPath := path.Join(idx.Index.Path(), "3", "L0-00000001.tsl")
+	tslFile, err := os.OpenFile(tslPath, os.O_RDWR, 0666)
+	require.NoError(t, err)
+	require.NoError(t, tslFile.Truncate(0))
+	// write poisonous TSL file - first byte doesn't matter, remaining bytes are an invalid uvarint
+	_, err = tslFile.Write([]byte{0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff})
+	require.NoError(t, err)
+	require.NoError(t, tslFile.Close())
+	idx.Index = tsi1.NewIndex(idx.SeriesFile.SeriesFile, "db0", tsi1.WithPath(idx.Index.Path()))
+	err = idx.Index.Open()
+	require.Error(t, err, "expected an error on opening the index")
+	require.Contains(t, err.Error(), ".tsl\": parsing binary-encoded uint64 value failed; binary.Uvarint() returned -11")
+	// ensure each partition is closed:
+	for i := 0; i < int(idx.Index.PartitionN); i++ {
+		assert.Equal(t, idx.Index.PartitionAt(i).FileN(), 0)
+	}
+	require.NoError(t, idx.Close())
+}
+
 func TestIndex_Open(t *testing.T) {
-	// Opening a fresh index should set the MANIFEST version to current version.
-	idx := NewDefaultIndex()
 	t.Run("open new index", func(t *testing.T) {
-		if err := idx.Open(); err != nil {
-			t.Fatal(err)
-		}
+		// Opening a fresh index should set the MANIFEST version to current version.
+		idx := MustOpenDefaultIndex(t)
+		t.Cleanup(func() { assert.NoError(t, idx.Close()) })
 
 		// Check version set appropriately.
 		for i := 0; uint64(i) < tsi1.DefaultPartitionN; i++ {
 			partition := idx.PartitionAt(i)
+
 			if got, exp := partition.Manifest().Version, 1; got != exp {
 				t.Fatalf("got index version %d, expected %d", got, exp)
 			}
 		}
+
+		for i := 0; i < int(idx.PartitionN); i++ {
+			p := idx.PartitionAt(i)
+
+			if got, exp := p.NeedsCompaction(false), false; got != exp {
+				t.Fatalf("got needs compaction %v, expected %v", got, exp)
+			}
+		}
 	})
 
 	// Reopening an open index should return an error.
 	t.Run("reopen open index", func(t *testing.T) {
+		idx := MustOpenDefaultIndex(t)
+		t.Cleanup(func() { assert.NoError(t, idx.Close()) })
+
+		// Manually closing the existing SeriesFile so that it won't be left
+		// opened after idx.Open(), which calls another idx.SeriesFile.Open().
+		//
+		// This is required for t.TempDir() to be cleaned-up successfully on
+		// Windows.
+		assert.NoError(t, idx.SeriesFile.Close())
+
 		err := idx.Open()
 		if err == nil {
-			idx.Close()
 			t.Fatal("didn't get an error on reopen, but expected one")
 		}
-		idx.Close()
 	})
 
 	// Opening an incompatible index should return an error.
 	incompatibleVersions := []int{-1, 0, 2}
 	for _, v := range incompatibleVersions {
 		t.Run(fmt.Sprintf("incompatible index version: %d", v), func(t *testing.T) {
-			idx = NewDefaultIndex()
+			idx := NewDefaultIndex(t)
+
 			// Manually create a MANIFEST file for an incompatible index version.
 			// under one of the partitions.
 			partitionPath := filepath.Join(idx.Path(), "2")
@@ -253,7 +298,7 @@ func TestIndex_Open(t *testing.T) {
 			}
 
 			// Log the MANIFEST file.
-			data, err := ioutil.ReadFile(mpath)
+			data, err := os.ReadFile(mpath)
 			if err != nil {
 				panic(err)
 			}
@@ -262,8 +307,8 @@ func TestIndex_Open(t *testing.T) {
 			// Opening this index should return an error because the MANIFEST has an
 			// incompatible version.
 			err = idx.Open()
-			if err != tsi1.ErrIncompatibleVersion {
-				idx.Close()
+			t.Cleanup(func() { assert.NoError(t, idx.Close()) })
+			if !errors.Is(err, tsi1.ErrIncompatibleVersion) {
 				t.Fatalf("got error %v, expected %v", err, tsi1.ErrIncompatibleVersion)
 			}
 		})
@@ -272,7 +317,7 @@ func TestIndex_Open(t *testing.T) {
 
 func TestIndex_Manifest(t *testing.T) {
 	t.Run("current MANIFEST", func(t *testing.T) {
-		idx := MustOpenIndex(tsi1.DefaultPartitionN)
+		idx := MustOpenIndex(t, tsi1.DefaultPartitionN)
 
 		// Check version set appropriately.
 		for i := 0; uint64(i) < tsi1.DefaultPartitionN; i++ {
@@ -281,11 +326,13 @@ func TestIndex_Manifest(t *testing.T) {
 				t.Fatalf("got MANIFEST version %d, expected %d", got, exp)
 			}
 		}
+
+		require.NoError(t, idx.Close())
 	})
 }
 
 func TestIndex_DiskSizeBytes(t *testing.T) {
-	idx := MustOpenIndex(tsi1.DefaultPartitionN)
+	idx := MustOpenIndex(t, tsi1.DefaultPartitionN)
 	defer idx.Close()
 
 	// Add series to index.
@@ -298,24 +345,34 @@ func TestIndex_DiskSizeBytes(t *testing.T) {
 		t.Fatal(err)
 	}
 
-	// Verify on disk size is the same in each stage.
-	// Each series stores flag(1) + series(uvarint(2)) + len(name)(1) + len(key)(1) + len(value)(1) + checksum(4).
-	expSize := int64(4 * 9)
+	idx.RunStateAware(t, func(t *testing.T, state int) {
+		// Each MANIFEST file is 419 bytes and there are tsi1.DefaultPartitionN of them
+		expSize := int64(tsi1.DefaultPartitionN * 419)
+		switch state {
+		case Initial:
+			fallthrough
+		case Reopen:
+			// In the log file, each series stores flag(1) + series(uvarint(2)) + len(name)(1) + len(key)(1) + len(value)(1) + checksum(4).
+			expSize += 4 * 9
+		case PostCompaction:
+			fallthrough
+		case PostCompactionReopen:
+			// For TSI files after a compaction, instead of 4*9, we have encoded measurement names, tag names, etc which is larger
+			expSize += 2202
+		}
 
-	// Each MANIFEST file is 419 bytes and there are tsi1.DefaultPartitionN of them
-	expSize += int64(tsi1.DefaultPartitionN * 419)
-
-	idx.Run(t, func(t *testing.T) {
 		if got, exp := idx.DiskSizeBytes(), expSize; got != exp {
+			// We had some odd errors - if the size is unexpected, log it
+			idx.Index.LogDiskSize(t)
 			t.Fatalf("got %d bytes, expected %d", got, exp)
 		}
 	})
 }
 
 func TestIndex_TagValueSeriesIDIterator(t *testing.T) {
-	idx1 := MustOpenDefaultIndex() // Uses the single series creation method CreateSeriesIfNotExists
+	idx1 := MustOpenDefaultIndex(t) // Uses the single series creation method CreateSeriesIfNotExists
 	defer idx1.Close()
-	idx2 := MustOpenDefaultIndex() // Uses the batch series creation method CreateSeriesListIfNotExists
+	idx2 := MustOpenDefaultIndex(t) // Uses the batch series creation method CreateSeriesListIfNotExists
 	defer idx2.Close()
 
 	// Add some series.
@@ -466,69 +523,6 @@ func TestIndex_TagValueSeriesIDIterator(t *testing.T) {
 	})
 }
 
-func TestIndex_DropSeriesList(t *testing.T) {
-	idx := MustOpenDefaultIndex() // Uses the single series creation method CreateSeriesIfNotExists
-	defer idx.Close()
-
-	// Add some series.
-	data := []struct {
-		Key  string
-		Name string
-		Tags map[string]string
-	}{
-		{"cpu,region=west,server=a", "cpu", map[string]string{"region": "west", "server": "a"}},
-		{"cpu,region=west,server=b", "cpu", map[string]string{"region": "west", "server": "b"}},
-		{"cpu,region=west,server=c", "cpu", map[string]string{"region": "west", "server": "c"}},
-		{"cpu,region=east,server=a", "cpu", map[string]string{"region": "east", "server": "a"}},
-		{"cpu,region=east,server=c", "cpu", map[string]string{"region": "east", "server": "c"}},
-		{"cpu,region=east,server=d", "cpu", map[string]string{"region": "east", "server": "d"}},
-		{"cpu,region=north,server=b", "cpu", map[string]string{"region": "north", "server": "b"}},
-		{"cpu,region=north,server=c", "cpu", map[string]string{"region": "north", "server": "c"}},
-		{"cpu,region=north,server=d", "cpu", map[string]string{"region": "north", "server": "d"}},
-		{"cpu,region=south,server=a", "cpu", map[string]string{"region": "south", "server": "a"}},
-		{"cpu,region=south,server=d", "cpu", map[string]string{"region": "south", "server": "d"}},
-	}
-
-	keys := make([][]byte, 0, 15)
-	seriesIDs := make([]uint64, 0, 15)
-	for _, pt := range data {
-		if err := idx.CreateSeriesIfNotExists([]byte(pt.Key), []byte(pt.Name), models.NewTags(pt.Tags)); err != nil {
-			t.Fatal(err)
-		}
-
-		keys = append(keys, []byte(pt.Key))
-		seriesIDs = append(seriesIDs, idx.Index.SeriesFile().SeriesID([]byte(pt.Name), models.NewTags(pt.Tags), nil))
-	}
-
-	// Drop series list
-	if err := idx.DropSeriesList(seriesIDs[0:len(seriesIDs)-2], keys[0:len(keys)-2], false); err != nil {
-		t.Fatal(err)
-	}
-
-	// Verify series still exists.
-	idx.Run(t, func(t *testing.T) {
-		if v, err := idx.MeasurementHasSeries([]byte("cpu")); err != nil {
-			t.Fatal(err)
-		} else if !v {
-			t.Fatal("expected series to still exist")
-		}
-	})
-
-	// Drop series list lefted
-	if err := idx.DropSeriesList(seriesIDs[len(seriesIDs)-2:], keys[len(keys)-2:], false); err != nil {
-		t.Fatal(err)
-	}
-
-	// Verify series is now deleted.
-	idx.Run(t, func(t *testing.T) {
-		if v, err := idx.MeasurementHasSeries([]byte("cpu")); err != nil {
-			t.Fatal(err)
-		} else if v {
-			t.Fatal("expected series to be deleted")
-		}
-	})
-}
-
 // Index is a test wrapper for tsi1.Index.
 type Index struct {
 	*tsi1.Index
@@ -536,21 +530,21 @@ type Index struct {
 }
 
 // NewIndex returns a new instance of Index at a temporary path.
-func NewIndex(partitionN uint64) *Index {
-	idx := &Index{SeriesFile: NewSeriesFile()}
-	idx.Index = tsi1.NewIndex(idx.SeriesFile.SeriesFile, "db0", tsi1.WithPath(MustTempDir()))
+func NewIndex(tb testing.TB, partitionN uint64) *Index {
+	idx := &Index{SeriesFile: NewSeriesFile(tb)}
+	idx.Index = tsi1.NewIndex(idx.SeriesFile.SeriesFile, "db0", tsi1.WithPath(tb.TempDir()))
 	idx.Index.PartitionN = partitionN
 	return idx
 }
 
 // NewIndex returns a new instance of Index with default number of partitions at a temporary path.
-func NewDefaultIndex() *Index {
-	return NewIndex(tsi1.DefaultPartitionN)
+func NewDefaultIndex(tb testing.TB) *Index {
+	return NewIndex(tb, tsi1.DefaultPartitionN)
 }
 
 // MustOpenIndex returns a new, open index. Panic on error.
-func MustOpenIndex(partitionN uint64) *Index {
-	idx := NewIndex(partitionN)
+func MustOpenIndex(tb testing.TB, partitionN uint64) *Index {
+	idx := NewIndex(tb, partitionN)
 	if err := idx.Open(); err != nil {
 		panic(err)
 	}
@@ -558,8 +552,8 @@ func MustOpenIndex(partitionN uint64) *Index {
 }
 
 // MustOpenIndex returns a new, open index with the default number of partitions.
-func MustOpenDefaultIndex() *Index {
-	return MustOpenIndex(tsi1.DefaultPartitionN)
+func MustOpenDefaultIndex(tb testing.TB) *Index {
+	return MustOpenIndex(tb, tsi1.DefaultPartitionN)
 }
 
 // Open opens the underlying tsi1.Index and tsdb.SeriesFile
@@ -572,15 +566,18 @@ func (idx Index) Open() error {
 
 // Close closes and removes the index directory.
 func (idx *Index) Close() error {
-	defer os.RemoveAll(idx.Path())
+	// Series file is opened first and must be closed last
+	if err := idx.Index.Close(); err != nil {
+		return err
+	}
 	if err := idx.SeriesFile.Close(); err != nil {
 		return err
 	}
-	return idx.Index.Close()
+	return nil
 }
 
 // Reopen closes and opens the index.
-func (idx *Index) Reopen() error {
+func (idx *Index) Reopen(maxLogSize int64) error {
 	if err := idx.Index.Close(); err != nil {
 		return err
 	}
@@ -592,11 +589,24 @@ func (idx *Index) Reopen() error {
 	}
 
 	partitionN := idx.Index.PartitionN // Remember how many partitions to use.
-	idx.Index = tsi1.NewIndex(idx.SeriesFile.SeriesFile, "db0", tsi1.WithPath(idx.Index.Path()))
+	idx.Index = tsi1.NewIndex(idx.SeriesFile.SeriesFile, "db0", tsi1.WithPath(idx.Index.Path()), tsi1.WithMaximumLogFileSize(maxLogSize))
 	idx.Index.PartitionN = partitionN
 	return idx.Open()
 }
 
+const (
+	Initial = iota
+	Reopen
+	PostCompaction
+	PostCompactionReopen
+)
+
+func curryState(state int, f func(t *testing.T, state int)) func(t *testing.T) {
+	return func(t *testing.T) {
+		f(t, state)
+	}
+}
+
 // Run executes a subtest for each of several different states:
 //
 // - Immediately
@@ -607,27 +617,42 @@ func (idx *Index) Reopen() error {
 // The index should always respond in the same fashion regardless of
 // how data is stored. This helper allows the index to be easily tested
 // in all major states.
-func (idx *Index) Run(t *testing.T, fn func(t *testing.T)) {
+func (idx *Index) RunStateAware(t *testing.T, fn func(t *testing.T, state int)) {
 	// Invoke immediately.
-	t.Run("state=initial", fn)
+	t.Run("state=initial", curryState(Initial, fn))
 
 	// Reopen and invoke again.
-	if err := idx.Reopen(); err != nil {
+	if err := idx.Reopen(tsdb.DefaultMaxIndexLogFileSize); err != nil {
 		t.Fatalf("reopen error: %s", err)
 	}
-	t.Run("state=reopen", fn)
+	t.Run("state=reopen", curryState(Reopen, fn))
 
-	// TODO: Request a compaction.
-	// if err := idx.Compact(); err != nil {
-	// 	t.Fatalf("compact error: %s", err)
-	// }
-	// t.Run("state=post-compaction", fn)
+	// Reopen requiring a full compaction of the TSL files and invoke again.
+	idx.Reopen(1)
+	for {
+		needsCompaction := false
+		for i := 0; i < int(idx.PartitionN); i++ {
+			needsCompaction = needsCompaction || idx.PartitionAt(i).NeedsCompaction(false)
+		}
+		if !needsCompaction {
+			break
+		}
+		time.Sleep(10 * time.Millisecond)
+	}
+	t.Run("state=post-compaction", curryState(PostCompaction, fn))
 
 	// Reopen and invoke again.
-	if err := idx.Reopen(); err != nil {
+	if err := idx.Reopen(tsdb.DefaultMaxIndexLogFileSize); err != nil {
 		t.Fatalf("post-compaction reopen error: %s", err)
 	}
-	t.Run("state=post-compaction-reopen", fn)
+	t.Run("state=post-compaction-reopen", curryState(PostCompactionReopen, fn))
+}
+
+// Run is the same is RunStateAware but for tests that do not depend on compaction state
+func (idx *Index) Run(t *testing.T, fn func(t *testing.T)) {
+	idx.RunStateAware(t, func(t *testing.T, _ int) {
+		fn(t)
+	})
 }
 
 // CreateSeriesSliceIfNotExists creates multiple series at a time.
@@ -656,7 +681,7 @@ var tsiditr tsdb.SeriesIDIterator
 func BenchmarkIndex_IndexFile_TagValueSeriesIDIterator(b *testing.B) {
 	runBenchMark := func(b *testing.B, cacheSize int) {
 		var err error
-		sfile := NewSeriesFile()
+		sfile := NewSeriesFile(b)
 		// Load index
 		idx := tsi1.NewIndex(sfile.SeriesFile, "foo",
 			tsi1.WithPath("testdata/index-file-index"),
@@ -730,7 +755,7 @@ func BenchmarkIndex_CreateSeriesListIfNotExists(b *testing.B) {
 		b.Fatal(err)
 	}
 
-	data, err := ioutil.ReadAll(gzr)
+	data, err := io.ReadAll(gzr)
 	if err != nil {
 		b.Fatal(err)
 	}
@@ -756,7 +781,7 @@ func BenchmarkIndex_CreateSeriesListIfNotExists(b *testing.B) {
 		b.Run(fmt.Sprintf("batch size %d", sz), func(b *testing.B) {
 			for _, partition := range partitions {
 				b.Run(fmt.Sprintf("partition %d", partition), func(b *testing.B) {
-					idx := MustOpenIndex(partition)
+					idx := MustOpenIndex(b, partition)
 					for j := 0; j < b.N; j++ {
 						for i := 0; i < len(keys); i += sz {
 							k := keys[i : i+sz]
@@ -771,7 +796,7 @@ func BenchmarkIndex_CreateSeriesListIfNotExists(b *testing.B) {
 						if err := idx.Close(); err != nil {
 							b.Fatal(err)
 						}
-						idx = MustOpenIndex(partition)
+						idx = MustOpenIndex(b, partition)
 						b.StartTimer()
 					}
 				})
@@ -811,7 +836,7 @@ func BenchmarkIndex_ConcurrentWriteQuery(b *testing.B) {
 		b.Fatal(err)
 	}
 
-	data, err := ioutil.ReadAll(gzr)
+	data, err := io.ReadAll(gzr)
 	if err != nil {
 		b.Fatal(err)
 	}
@@ -832,8 +857,8 @@ func BenchmarkIndex_ConcurrentWriteQuery(b *testing.B) {
 	}
 
 	runBenchmark := func(b *testing.B, queryN int, partitions uint64, cacheSize int) {
-		idx := &Index{SeriesFile: NewSeriesFile()}
-		idx.Index = tsi1.NewIndex(idx.SeriesFile.SeriesFile, "db0", tsi1.WithPath(MustTempDir()), tsi1.WithSeriesIDCacheSize(cacheSize))
+		idx := &Index{SeriesFile: NewSeriesFile(b)}
+		idx.Index = tsi1.NewIndex(idx.SeriesFile.SeriesFile, "db0", tsi1.WithPath(b.TempDir()), tsi1.WithSeriesIDCacheSize(cacheSize))
 		idx.Index.PartitionN = partitions
 
 		if err := idx.Open(); err != nil {
@@ -891,8 +916,8 @@ func BenchmarkIndex_ConcurrentWriteQuery(b *testing.B) {
 			}
 
 			// Re-open everything
-			idx := &Index{SeriesFile: NewSeriesFile()}
-			idx.Index = tsi1.NewIndex(idx.SeriesFile.SeriesFile, "db0", tsi1.WithPath(MustTempDir()), tsi1.WithSeriesIDCacheSize(cacheSize))
+			idx := &Index{SeriesFile: NewSeriesFile(b)}
+			idx.Index = tsi1.NewIndex(idx.SeriesFile.SeriesFile, "db0", tsi1.WithPath(b.TempDir()), tsi1.WithSeriesIDCacheSize(cacheSize))
 			idx.Index.PartitionN = partitions
 
 			if err := idx.Open(); err != nil {
