@@ -1,3 +1,4 @@
+//lint:file-ignore SA5011 we use assertions, which don't guard
 package tsi1
 
 import (
@@ -14,12 +15,12 @@ import (
 	"time"
 	"unsafe"
 
-	"github.com/influxdata/influxdb/models"
-	"github.com/influxdata/influxdb/pkg/bloom"
-	"github.com/influxdata/influxdb/pkg/estimator"
-	"github.com/influxdata/influxdb/pkg/estimator/hll"
-	"github.com/influxdata/influxdb/pkg/mmap"
-	"github.com/influxdata/influxdb/tsdb"
+	"github.com/influxdata/influxdb/v2/models"
+	"github.com/influxdata/influxdb/v2/pkg/bloom"
+	"github.com/influxdata/influxdb/v2/pkg/estimator"
+	"github.com/influxdata/influxdb/v2/pkg/estimator/hll"
+	"github.com/influxdata/influxdb/v2/pkg/mmap"
+	"github.com/influxdata/influxdb/v2/tsdb"
 )
 
 // Log errors.
@@ -86,6 +87,8 @@ func NewLogFile(sfile *tsdb.SeriesFile, path string) *LogFile {
 
 // bytes estimates the memory footprint of this LogFile, in bytes.
 func (f *LogFile) bytes() int {
+	f.mu.RLock()
+	defer f.mu.RUnlock()
 	var b int
 	b += 24 // mu RWMutex is 24 bytes
 	b += 16 // wg WaitGroup is 16 bytes
@@ -107,6 +110,8 @@ func (f *LogFile) bytes() int {
 
 // Open reads the log from a file and validates all the checksums.
 func (f *LogFile) Open() error {
+	f.mu.Lock()
+	defer f.mu.Unlock()
 	if err := f.open(); err != nil {
 		f.Close()
 		return err
@@ -151,10 +156,10 @@ func (f *LogFile) open() error {
 	for buf := f.data; len(buf) > 0; {
 		// Read next entry. Truncate partial writes.
 		var e LogEntry
-		if err := e.UnmarshalBinary(buf); err == io.ErrShortBuffer || err == ErrLogEntryChecksumMismatch {
+		if err := e.UnmarshalBinary(buf); errors.Is(err, io.ErrShortBuffer) || errors.Is(err, ErrLogEntryChecksumMismatch) {
 			break
 		} else if err != nil {
-			return err
+			return fmt.Errorf("%q: %w", f.path, err)
 		}
 
 		// Execute entry against in-memory index.
@@ -260,6 +265,13 @@ func (f *LogFile) Size() int64 {
 	return v
 }
 
+// ModTime returns the last modified time of the file
+func (f *LogFile) ModTime() time.Time {
+	f.mu.RLock()
+	defer f.mu.RUnlock()
+	return f.modTime
+}
+
 // Measurement returns a measurement element.
 func (f *LogFile) Measurement(name []byte) MeasurementElem {
 	f.mu.RLock()
@@ -282,13 +294,7 @@ func (f *LogFile) MeasurementHasSeries(ss *tsdb.SeriesIDSet, name []byte) bool {
 		return false
 	}
 
-	// TODO(edd): if mm is using a seriesSet then this could be changed to do a fast intersection.
-	for _, id := range mm.seriesIDs() {
-		if ss.Contains(id) {
-			return true
-		}
-	}
-	return false
+	return mm.hasSeries(ss)
 }
 
 // MeasurementNames returns an ordered list of measurement names.
@@ -323,18 +329,18 @@ func (f *LogFile) DeleteMeasurement(name []byte) error {
 }
 
 // TagKeySeriesIDIterator returns a series iterator for a tag key.
-func (f *LogFile) TagKeySeriesIDIterator(name, key []byte) tsdb.SeriesIDIterator {
+func (f *LogFile) TagKeySeriesIDIterator(name, key []byte) (tsdb.SeriesIDIterator, error) {
 	f.mu.RLock()
 	defer f.mu.RUnlock()
 
 	mm, ok := f.mms[string(name)]
 	if !ok {
-		return nil
+		return nil, nil
 	}
 
 	tk, ok := mm.tagSet[string(key)]
 	if !ok {
-		return nil
+		return nil, nil
 	}
 
 	// Combine iterators across all tag keys.
@@ -348,7 +354,7 @@ func (f *LogFile) TagKeySeriesIDIterator(name, key []byte) tsdb.SeriesIDIterator
 		}
 	}
 
-	return tsdb.MergeSeriesIDIterators(itrs...)
+	return tsdb.MergeSeriesIDIterators(itrs...), nil
 }
 
 // TagKeyIterator returns a value iterator for a measurement.
@@ -365,7 +371,7 @@ func (f *LogFile) TagKeyIterator(name []byte) TagKeyIterator {
 	for _, k := range mm.tagSet {
 		a = append(a, k)
 	}
-	return newLogTagKeyIterator(a)
+	return newLogTagKeyIterator(f, a)
 }
 
 // TagKey returns a tag key element.
@@ -441,29 +447,29 @@ func (f *LogFile) DeleteTagKey(name, key []byte) error {
 	return f.FlushAndSync()
 }
 
-// TagValueSeriesIDIterator returns a series iterator for a tag value.
-func (f *LogFile) TagValueSeriesIDIterator(name, key, value []byte) tsdb.SeriesIDIterator {
+// TagValueSeriesIDSet returns a series iterator for a tag value.
+func (f *LogFile) TagValueSeriesIDSet(name, key, value []byte) (*tsdb.SeriesIDSet, error) {
 	f.mu.RLock()
 	defer f.mu.RUnlock()
 
 	mm, ok := f.mms[string(name)]
 	if !ok {
-		return nil
+		return nil, nil
 	}
 
 	tk, ok := mm.tagSet[string(key)]
 	if !ok {
-		return nil
+		return nil, nil
 	}
 
 	tv, ok := tk.tagValues[string(value)]
 	if !ok {
-		return nil
+		return nil, nil
 	} else if tv.cardinality() == 0 {
-		return nil
+		return nil, nil
 	}
 
-	return tsdb.NewSeriesIDSetIterator(tv.seriesIDSet())
+	return tv.seriesIDSet(), nil
 }
 
 // MeasurementN returns the total number of measurements.
@@ -511,10 +517,10 @@ func (f *LogFile) DeleteTagValue(name, key, value []byte) error {
 }
 
 // AddSeriesList adds a list of series to the log file in bulk.
-func (f *LogFile) AddSeriesList(seriesSet *tsdb.SeriesIDSet, names [][]byte, tagsSlice []models.Tags) error {
+func (f *LogFile) AddSeriesList(seriesSet *tsdb.SeriesIDSet, names [][]byte, tagsSlice []models.Tags) ([]uint64, error) {
 	seriesIDs, err := f.sfile.CreateSeriesListIfNotExists(names, tagsSlice)
 	if err != nil {
-		return err
+		return nil, err
 	}
 
 	var writeRequired bool
@@ -523,16 +529,17 @@ func (f *LogFile) AddSeriesList(seriesSet *tsdb.SeriesIDSet, names [][]byte, tag
 	for i := range names {
 		if seriesSet.ContainsNoLock(seriesIDs[i]) {
 			// We don't need to allocate anything for this series.
+			seriesIDs[i] = 0
 			continue
 		}
 		writeRequired = true
-		entries = append(entries, LogEntry{SeriesID: seriesIDs[i], name: names[i], tags: tagsSlice[i], cached: true})
+		entries = append(entries, LogEntry{SeriesID: seriesIDs[i], name: names[i], tags: tagsSlice[i], cached: true, batchidx: i})
 	}
 	seriesSet.RUnlock()
 
 	// Exit if all series already exist.
 	if !writeRequired {
-		return nil
+		return seriesIDs, nil
 	}
 
 	f.mu.Lock()
@@ -541,21 +548,25 @@ func (f *LogFile) AddSeriesList(seriesSet *tsdb.SeriesIDSet, names [][]byte, tag
 	seriesSet.Lock()
 	defer seriesSet.Unlock()
 
-	for i := range entries {
+	for i := range entries { // NB - this doesn't evaluate all series ids returned from series file.
 		entry := &entries[i]
 		if seriesSet.ContainsNoLock(entry.SeriesID) {
 			// We don't need to allocate anything for this series.
+			seriesIDs[entry.batchidx] = 0
 			continue
 		}
 		if err := f.appendEntry(entry); err != nil {
-			return err
+			return nil, err
 		}
 		f.execEntry(entry)
 		seriesSet.AddNoLock(entry.SeriesID)
 	}
 
 	// Flush buffer and sync to disk.
-	return f.FlushAndSync()
+	if err := f.FlushAndSync(); err != nil {
+		return nil, err
+	}
+	return seriesIDs, nil
 }
 
 // DeleteSeriesID adds a tombstone for a series id.
@@ -682,6 +693,9 @@ func (f *LogFile) execSeriesEntry(e *LogEntry) {
 
 	// Read key size.
 	_, remainder := tsdb.ReadSeriesKeyLen(seriesKey)
+	if len(remainder) == 0 {
+		return
+	}
 
 	// Read measurement name.
 	name, remainder := tsdb.ReadSeriesKeyMeasurement(remainder)
@@ -711,6 +725,7 @@ func (f *LogFile) execSeriesEntry(e *LogEntry) {
 		}
 
 		ts.tagValues[string(v)] = tv
+
 		mm.tagSet[string(k)] = ts
 	}
 
@@ -757,6 +772,7 @@ func (f *LogFile) createMeasurementIfNotExists(name []byte) *logMeasurement {
 	mm := f.mms[string(name)]
 	if mm == nil {
 		mm = &logMeasurement{
+			f:      f,
 			name:   name,
 			tagSet: make(map[string]logTagKey),
 			series: make(map[uint64]struct{}),
@@ -935,7 +951,7 @@ func (f *LogFile) writeTagsetTo(w io.Writer, name string, info *logFileCompactIn
 		// Add each value.
 		for _, v := range values {
 			value := tag.tagValues[v]
-			if err := enc.EncodeValue(value.name, value.deleted, value.seriesIDs()); err != nil {
+			if err := enc.EncodeValue(value.name, value.deleted, value.seriesIDSet()); err != nil {
 				return err
 			}
 
@@ -1046,7 +1062,7 @@ func (f *LogFile) seriesSketches() (sketch, tSketch estimator.Sketch, err error)
 	tSketch = hll.NewDefaultPlus()
 	f.tombstoneSeriesIDSet.ForEach(func(id uint64) {
 		name, keys := f.sfile.Series(id)
-		sketch.Add(models.MakeKey(name, keys))
+		tSketch.Add(models.MakeKey(name, keys))
 	})
 	return sketch, tSketch, nil
 }
@@ -1061,9 +1077,10 @@ type LogEntry struct {
 	Checksum uint32 // checksum of flag/name/tags.
 	Size     int    // total size of record, in bytes.
 
-	cached bool        // Hint to LogFile that series data is already parsed
-	name   []byte      // series naem, this is a cached copy of the parsed measurement name
-	tags   models.Tags // series tags, this is a cached copied of the parsed tags
+	cached   bool        // Hint to LogFile that series data is already parsed
+	name     []byte      // series naem, this is a cached copy of the parsed measurement name
+	tags     models.Tags // series tags, this is a cached copied of the parsed tags
+	batchidx int         // position of entry in batch.
 }
 
 // UnmarshalBinary unmarshals data into e.
@@ -1194,6 +1211,7 @@ func (mms *logMeasurements) bytes() int {
 }
 
 type logMeasurement struct {
+	f         *LogFile
 	name      []byte
 	tagSet    map[string]logTagKey
 	deleted   bool
@@ -1288,13 +1306,27 @@ func (m *logMeasurement) seriesIDSet() *tsdb.SeriesIDSet {
 	return ss
 }
 
+func (m *logMeasurement) hasSeries(ss *tsdb.SeriesIDSet) bool {
+	if m.seriesSet != nil {
+		return m.seriesSet.Intersects(ss)
+	}
+
+	for seriesID := range m.series {
+		if ss.Contains(seriesID) {
+			return true
+		}
+	}
+
+	return false
+}
+
 func (m *logMeasurement) Name() []byte  { return m.name }
 func (m *logMeasurement) Deleted() bool { return m.deleted }
 
 func (m *logMeasurement) createTagSetIfNotExists(key []byte) logTagKey {
 	ts, ok := m.tagSet[string(key)]
 	if !ok {
-		ts = logTagKey{name: key, tagValues: make(map[string]logTagValue)}
+		ts = logTagKey{f: m.f, name: key, tagValues: make(map[string]logTagValue)}
 	}
 	return ts
 }
@@ -1331,6 +1363,7 @@ func (itr *logMeasurementIterator) Next() (e MeasurementElem) {
 }
 
 type logTagKey struct {
+	f         *LogFile
 	name      []byte
 	deleted   bool
 	tagValues map[string]logTagValue
@@ -1352,10 +1385,13 @@ func (tk *logTagKey) Key() []byte   { return tk.name }
 func (tk *logTagKey) Deleted() bool { return tk.deleted }
 
 func (tk *logTagKey) TagValueIterator() TagValueIterator {
+	tk.f.mu.RLock()
 	a := make([]logTagValue, 0, len(tk.tagValues))
 	for _, v := range tk.tagValues {
 		a = append(a, v)
 	}
+	tk.f.mu.RUnlock()
+
 	return newLogTagValueIterator(a)
 }
 
@@ -1423,21 +1459,6 @@ func (tv *logTagValue) cardinality() int64 {
 	return int64(len(tv.series))
 }
 
-// seriesIDs returns a sorted set of seriesIDs.
-func (tv *logTagValue) seriesIDs() []uint64 {
-	a := make([]uint64, 0, tv.cardinality())
-	if tv.seriesSet != nil {
-		tv.seriesSet.ForEachNoLock(func(id uint64) { a = append(a, id) })
-		return a // IDs are already sorted.
-	}
-
-	for seriesID := range tv.series {
-		a = append(a, seriesID)
-	}
-	sort.Sort(uint64Slice(a))
-	return a
-}
-
 // seriesIDSet returns a copy of the logMeasurement's seriesSet, or creates a new
 // one
 func (tv *logTagValue) seriesIDSet() *tsdb.SeriesIDSet {
@@ -1464,13 +1485,14 @@ func (a logTagValueSlice) Less(i, j int) bool { return bytes.Compare(a[i].name,
 
 // logTagKeyIterator represents an iterator over a slice of tag keys.
 type logTagKeyIterator struct {
+	f *LogFile
 	a []logTagKey
 }
 
 // newLogTagKeyIterator returns a new instance of logTagKeyIterator.
-func newLogTagKeyIterator(a []logTagKey) *logTagKeyIterator {
+func newLogTagKeyIterator(f *LogFile, a []logTagKey) *logTagKeyIterator {
 	sort.Sort(logTagKeySlice(a))
-	return &logTagKeyIterator{a: a}
+	return &logTagKeyIterator{f: f, a: a}
 }
 
 // Next returns the next element in the iterator.
